// =============================
//  class DATA opens a datafile (txt file with Mocap location data)
//  and reads the data into a class Data used in the Viewer
// 
//  created by M. Geroch
//             May-June 2001
// =====================================

import java.awt.*;
import java.io.*;
import java.util.*;
import java.applet.*;
import java.net.*;
import javax.vecmath.*;
import javax.media.j3d.*;
import java.lang.*;


public class Data_asf {
  asf_struct[] joints;
  int number_joints;
  int frame_number;
  double max_p[],min_p[];
  float knots[];
    
  static boolean filled;
  static boolean amcfilled;
  String setname;
  String amcname;
  long secs;
  
  // ====================
  // constructors
  // ----------------------
  Data_asf() {
    setname = null; 
    filled = false;
	secs = 0;
	frame_number=0;
	max_p = new double[3];
	min_p = new double[3];
  }
  


  // ===========================
  // utilities
  // --------------------------
  public void export3d(String filename,int timer,Transform3D ttt)
  {
    int i,j;
	double factor = 0.7*0.025;
	try{
	File outputFile = new File(filename);
	FileWriter out1 = new FileWriter(outputFile);
	String temp="% Field Time ";
    for(i=0;i<number_joints;i++)
	{
		temp=temp+joints[i].name+":X "+joints[i].name+":Y "+joints[i].name+":Z ";
	}
	out1.write(temp+"\r");
    for(i=0;i<frame_number;i++)
	{
		temp=Integer.toString(i)+" "+Float.toString((float)(i*1.0/(float)timer))+" ";
		for(j=0;j<number_joints;j++)
		{
		    Point3d point = new Point3d(joints[j].amc_position[i][0],joints[j].amc_position[i][1],joints[j].amc_position[i][2]);
			ttt.transform(point);
			temp=temp+Double.toString(point.x/factor)+" "+Double.toString(point.y/factor)+" "+Double.toString(point.z/factor)+" ";
		}
		temp = temp+"\r";
	    out1.write(temp);
	}
	out1.close();
	}
	catch (Exception e) {
      System.out.println("Some problem in exporting 3d file");
      System.out.println(e.toString());
    }

	try{
	int index = filename.lastIndexOf(".");
	filename = filename.substring(0,index);
	filename = filename + ".hrc";
	File outputFile = new File(filename);
	FileWriter out1 = new FileWriter(outputFile);
	String temp="% Field Time ";
    for(i=0;i<number_joints;i++)
	{
		temp = joints[i].name;
		for( j=0;j<joints[i].number_son;j++)
			temp = temp + " " + joints[(joints[i]).sons[j]].name;	
		temp = temp+"\r";
		if(joints[i].number_son>0)out1.write(temp);
		temp = null;
	}
	
	out1.close();
	}
	catch (Exception e) {
      System.out.println("Some problem in exporting 3d file");
      System.out.println(e.toString());
    }
  }

  public void export2d(String filename,TimeCanvas3D c,Transform3D tr,int timer)
  {
    int i,j;
	try{
	File outputFile = new File(filename);
	FileWriter out1 = new FileWriter(outputFile);
	String temp="% Field Time ";
    for(i=0;i<number_joints;i++)
	{
		temp=temp+joints[i].name+":X "+joints[i].name+":Y ";
	}
	out1.write(temp+"\r");
	//System.out.println(timer);
    for(i=0;i<frame_number;i++)
	{
		temp=Integer.toString(i)+" "+Float.toString((float)(i*1.0/(float)timer))+" ";
		for(j=0;j<number_joints;j++)
		{
		    Point3d point = new Point3d(joints[j].amc_position[i][0],joints[j].amc_position[i][1],joints[j].amc_position[i][2]);
			tr.transform(point);
			Point2d pixelLocation = new Point2d();
			
			c.getPixelLocationFromImagePlate(point,pixelLocation);
			temp = temp+Double.toString(pixelLocation.x)+" ";
			temp = temp+Double.toString(pixelLocation.y)+" ";
			//temp = temp+Integer.toString((int)pos[2])+" ";
		}
		temp = temp+"\r";
	    out1.write(temp);
	}
	out1.close();
	}
	catch (Exception e) {
      System.out.println("Some problem in exporting 3d file");
      System.out.println(e.toString());
    }

	/*try{
	int index = filename.lastIndexOf(".");
	filename = filename.substring(0,index);
	filename = filename + ".hrc";
	File outputFile = new File(filename);
	FileWriter out1 = new FileWriter(outputFile);
	String temp="% Field Time ";
    for(i=0;i<number_joints;i++)
	{
		temp = joints[i].name;
		for( j=0;j<joints[i].number_son;j++)
			temp = temp + " " + joints[(joints[i]).sons[j]].name;	
		temp = temp+"\r";
		if(joints[i].number_son>0)out1.write(temp);
		temp = null;
	}
	
	out1.close();
	}
	catch (Exception e) {
      System.out.println("Some problem in exporting 3d file");
      System.out.println(e.toString());
    }*/
  }

  public int find(String name)
  {
	for(int i=0;i<number_joints;i++)
	{
		if((joints[i].name).equals(name))
		 return i;
	}
	return -1;
  }

  public void clearamcData(){
    if(amcname != null)amcname = null;
	amcfilled = false;
  }

  public void clearData() {
    if(setname != null)setname = null;
    filled = false;
  }
  
  public void setData(String movefile,String fn) {
    filled = false;
    try {
      getMovingInfo(movefile,fn);
    }
    catch (Exception e) {
      System.out.println("Some early problem in datafile: "+movefile);
      System.out.println(e.toString());
      System.out.println(" -- aborting cleanly");
      clearData();
    }
    if (filled) {
	calcDerivedData();
    }
  }

  public void gethrcMovingInfo(String filename) throws Exception {
    
    try {
	  File input = new File(filename);
	  System.out.println(input);
	  FileReader in = new FileReader(input);
	  BufferedReader dis = new BufferedReader(in);
	  String data;
	  
      String sn = new String(filename);
      int slashindex = sn.lastIndexOf('/');
      int dotindex = sn.lastIndexOf('.');
      setname = sn.substring(slashindex+1, dotindex);
	  System.out.println("setname ok");
      
      // read in first line, count nummarkers
      data = dis.readLine();
	    
      StringTokenizer st = new StringTokenizer(data);
	  
	  int nummarkers = (st.countTokens()-2)/3;
	  int numframes = 0;
	  int i;
	
	  // continue to read in lines, counting numframes
	  String[] savedata = new String[3000];
	  savedata[numframes] = dis.readLine();
	  while (savedata[numframes] != null) {
	    	numframes++;
		savedata[numframes] = dis.readLine();
	  }

	  joints = new asf_struct[nummarkers];
	  number_joints = nummarkers;
	  String toke;
	  for(i=0;i<number_joints;i++)
	  {
		 do{
		 toke = st.nextToken();
		 }while(!toke.endsWith(":X"));
		 int index = toke.indexOf(":");
		 toke = toke.substring(0,index);
	     joints[i]=new asf_struct();
		 joints[i].name=toke;
		 joints[i].amc_position = new float[numframes][3];
		 joints[i].pos_front_up = new float[numframes][3];
		 joints[i].pos_back_down = new float[numframes][3];
		 joints[i].pos_roottonode_left = new float[numframes][3];
		 joints[i].pos_roottonode_right = new float[numframes][3];
		 joints[i].frame_num = numframes;
	  }

	  // close the file
	  dis.close();
	  System.out.println("read done");
	
	  // now parse the saved data
	  float x, y, z,times;
	  int fr;
	  for (int frnum= 0; frnum < numframes; ++frnum) {
		st = new StringTokenizer(savedata[frnum]);
	    fr = Integer.parseInt(st.nextToken());  // discard frame number
	    times = Float.parseFloat(st.nextToken());
	    for (int mknum = 0; mknum < nummarkers; ++mknum) {
			x = (Float.parseFloat(st.nextToken()));//*0.025f;///1000.0f; 
			y = (Float.parseFloat(st.nextToken()));//*0.025f;///1000.0f; 
			z = (Float.parseFloat(st.nextToken()));//*0.025f;///1000.0f; 
			joints[mknum].amc_position[frnum][0] = x;
			joints[mknum].amc_position[frnum][1] = y;
			joints[mknum].amc_position[frnum][2] = z;
	  }
	}
	filled = true;
	amcfilled = true;	
    }
    catch (Exception e) {
      System.out.println("Some problem in data file -- aborting");
      System.out.println(e.toString());
      clearData();
    }
	System.out.println("done in t3d");
	try {
	  String toke;
	  int index = filename.lastIndexOf(".");
	  filename = filename.substring(0,index);
	  filename = filename + ".hrc";
	  File input = new File(filename);
	  System.out.println(input);
      FileReader in = new FileReader(input);

	  BufferedReader dis = new BufferedReader(in);
      String[] data;
	

	  data = new String[1000];
	  int index1;
      
	  int linecount = 0;
	
      data[linecount] = dis.readLine();

	  // need to find out the # of joints
	  while(data[linecount]!= null)
	  {
		 linecount++;
		 data[linecount] = dis.readLine();
	  }
	  // now parse the saved data
	  System.out.println("linecount:"+linecount);
	  int fr=0;
	  int i=0;
	  while(i<linecount){
	   while (data[i].equals(""))
		i++;	
	   StringTokenizer st = new StringTokenizer(data[i]);
	   toke = st.nextToken();
	   index = find(toke);
	   System.out.println(index);
	   index1 = st.countTokens();
	   System.out.println(index1);
	   joints[index].sons = new int[index1];
	   joints[index].number_son = index1;
	   int j=0;
       for(j=0;j<index1;j++)
		    joints[index].sons[j] = -1;
	   System.out.println("before while");
	   j=0;
	   while (st.hasMoreTokens()) 
	   {
			toke = st.nextToken();
			index1 = find(toke);
			joints[index].sons[j] = index1;
			j++;
	   }
	   i++;
	   }
    }
    catch (Exception e) {
      System.out.println("Some problem in data file -- aborting");
      System.out.println(e.toString());
      clearData();
    }
}

  public void sethrcData(String movefile,String fn) {
    filled = false;
    try {
      gethrcMovingInfo(movefile);
    }
    catch (Exception e) {
      System.out.println("Some early problem in datafile: "+movefile);
      System.out.println(e.toString());
      System.out.println(" -- aborting cleanly");
      clearData();
    }
  }

  public void setamcData(String movefile,String fn) {
    amcfilled = false;
    try {
      getamcMovingInfo(movefile,fn);
    }
    catch (Exception e) {
      System.out.println("Some early problem in datafile: "+movefile);
      System.out.println(e.toString());
      System.out.println(" -- aborting cleanly");
      clearData();
    }
    if (amcfilled) {
	calcDerivedamcData();
    //calcboundary();
    }
  }

  public void calcboundary()
  {
    int i,j;
	for(i=0;i<3;i++)
	{
		max_p[i] = -1000;
		min_p[i] = +1000;
	}
	for(i=0;i<number_joints;i++)
	{
		for(j=0;j<frame_number;j++)
		  {
		    for(int k=0;k<3;k++)
			{
				if(max_p[k]<joints[i].amc_position[j][k])max_p[k]=joints[i].amc_position[j][k];
				if(min_p[k]>joints[i].amc_position[j][k])min_p[k]=joints[i].amc_position[j][k];
			}
		  }
	}
	System.out.println(max_p[0]+" "+max_p[1]+" "+max_p[2]);
	System.out.println(min_p[0]+" "+min_p[1]+" "+min_p[2]);
  }
  
  public void getamcMovingInfo(String filename, String fn) throws Exception {
    int temp =0;
	int i,j,k;
	int index,index1;
	String toke;
	StringTokenizer st;
	try {
      File input = new File(filename);
	  FileReader in = new FileReader(input);
	  BufferedReader dis = new BufferedReader(in);
      int slashindex = fn.lastIndexOf('/');
	  int dotindex = fn.lastIndexOf('.');
	  String data;
	  
      
      amcname = fn.substring(slashindex+1, dotindex);


      data = dis.readLine();
	  // need to find out the # of joints
	  char num;
  
	  while(data!= null)
	  {
		 st = new StringTokenizer(data);
		 if(st.hasMoreTokens()) {
			toke = st.nextToken();
			num = toke.charAt(0);
		    if(num-'0'>0 && num-'9'<=0)temp++;
			}
		 data = dis.readLine();
	  }
	  //System.out.println("number of frame:"+Integer.toString(temp));
      for(i=0;i<number_joints;i++)
	  {
	    joints[i].dofc= new float[temp][joints[i].number_dof];
		joints[i].amc_position = new float[temp][3];
		joints[i].pos_front_up = new float[temp][3];
		joints[i].pos_back_down = new float[temp][3];
		joints[i].pos_roottonode_left = new float[temp][3];
		joints[i].pos_roottonode_right = new float[temp][3];
		joints[i].frame_num = temp;
	  }
	  knots = new float[temp];
	 dis.close();
     // now parse the file
	 int frame_num=0;	
	 
	 input = new File(filename);
	 in = new FileReader(input);
	 dis = new BufferedReader(in);
	 data = dis.readLine();
	 boolean found_joints = false;
	 while(data!= null)
	  {
		 st = new StringTokenizer(data);
		 if(st.hasMoreTokens()) {
			toke = st.nextToken();
			num = toke.charAt(0);
		    if(num-'0'>0 && num-'9'<=0)
			{
			    while(frame_num<=temp)
				{
					data = dis.readLine();
					if(data != null)
					{
					  st = new StringTokenizer(data);
					  toke = st.nextToken();
					  num = toke.charAt(0);
					  if(num-'0'>0 && num-'9'<=0)
					  {
					    frame_num++;
						continue;
					  }
					  found_joints = false;
					  for(i=0;i<number_joints;i++)
					  {
						 if(toke.equals(joints[i].name))
						 {
						    for(k=0;k<joints[i].number_dof;k++)
							  joints[i].dofc[frame_num][k]=Float.parseFloat(st.nextToken());
							found_joints = true;
							break;
						 }
					  }
					  if(!found_joints)return;
					}
					else
					  break;
				}
				
			}
		 }
		if(data != null)data = dis.readLine();
	  }
	frame_number = temp;
	dis.close();

	File outputFile = new File("debug1.txt");
	FileWriter out1 = new FileWriter(outputFile);
    for(i=0;i<frame_number;i++)
	{
	  out1.write("frame"+Integer.toString(i));
	  //fp.writeChars("\r");
	  out1.write("\r");
	  for(j=0;j<number_joints;j++)
	     {
			out1.write(joints[j].name);
			out1.write("\r");
			//fp.writeChars("\r");
			String temp1=null;
			for(k=0;k<6;k++)
			{
			    if(temp1==null)temp1=Integer.toString(joints[j].dof[k]);
				else
				   temp1= temp1+' '+Integer.toString(joints[j].dof[k]);
			}
			out1.write(temp1);
			out1.write("\r");
			//fp.writeChars("\r");
			temp1 = null;
			for(k=0;k<joints[j].number_dof;k++)
			{
			    if(temp1==null)temp1=Float.toString(joints[j].dofc[i][k]);
				else
					temp1=temp1+" "+Float.toString(joints[j].dofc[i][k]);
				//fp.writeChars();
				//fp.writeChar(',');
			}
			if(temp1!=null)
			{ 
			   out1.write(temp1);
			   out1.write("\r");
			}
			//fp.writeChars("\r");
		 }
	  out1.write("/*******************************************************************/\r");
    }
	out1.close();
    amcfilled = true;
    }
    catch (Exception e) {
      System.out.println("Some problem in data file -- aborting");
      System.out.println(e.toString());
      clearData();
    }
  }

  // ============================
  // getInfo -- opens file, reads in data, fills attributes
  // ---------------------------------
  public void getMovingInfo(String filename,String fn) throws Exception {
    int temp =0;
	int linecount =0;
	int i;
	int index,index1;
	String toke;
	StringTokenizer st;
    try {

	  File input = new File(filename);
	  System.out.println(input);
      FileReader in = new FileReader(input);

	  BufferedReader dis = new BufferedReader(in);
      String[] data;
	

	  data = new String[1000];
	  int slashindex = fn.lastIndexOf('/');
	  int dotindex = fn.lastIndexOf('.');
      setname = fn.substring(slashindex+1, dotindex);
	
  data[linecount] = dis.readLine();

	  // need to find out the # of joints
	  while(data[linecount]!= null)
	  {
		 st = new StringTokenizer(data[linecount]);
		 linecount++;
	     while (st.hasMoreTokens()) {
			toke = st.nextToken();
			if(toke.equals("id"))
				temp++;
			}
		 data[linecount] = dis.readLine();
	  }

     joints = new asf_struct[temp+1];
	 number_joints = temp+1;
	 for(i=0;i<number_joints;i++)
	    joints[i]=new asf_struct();
	 temp = 0;
	 dis.close();
	
	
	System.out.println(linecount);
	// now parse the saved data
	int fr=0;
	i=0;
	while(i<linecount){
	  while (data[i].equals(""))
		i++;	
	  st = new StringTokenizer(data[i]);
	  //System.out.println(i);
	  //System.out.println(data[i]);
	  toke = st.nextToken();
	  if(toke.equals(":root"))
	  {
	     //System.out.println("in root");
		 //System.out.println(temp);
		 //System.out.println(joints[temp].name);
	     joints[temp].name = "root";
		 joints[temp].root = 1;
		 //System.out.println("in root2");
		 i++;
		while ((data[i].trim()).equals(""))
  		  i++;
		 st = new StringTokenizer(data[i]);
		 toke = st.nextToken();
		 while(!(toke.equals(":bonedata")))
		 {
			if(toke.equals("order"))
			 {
			    while (st.hasMoreTokens()) 
				{
					toke = st.nextToken();
					if(toke.equals("TX"))
						joints[temp].dof[fr]=1;
					else if(toke.equals("TY"))
						joints[temp].dof[fr]=2;
					else if(toke.equals("TZ"))
						joints[temp].dof[fr]=3;
					else if(toke.equals("RZ"))
						joints[temp].dof[fr]=4;
					else if(toke.equals("RY"))
						joints[temp].dof[fr]=5;
					else if(toke.equals("RX"))
						joints[temp].dof[fr]=6;
					fr++;
				}
				joints[temp].number_dof = fr;
				fr = 0;
			 }
			else if(toke.equals("position"))
			 {
			    while (st.hasMoreTokens()) 
				{
					joints[temp].position[0]=Float.parseFloat(st.nextToken());
					joints[temp].position[1]=Float.parseFloat(st.nextToken());
					joints[temp].position[2]=Float.parseFloat(st.nextToken());
				}
			 }
			else if(toke.equals("orientation"))
			 {
			    while (st.hasMoreTokens()) 
				{
					joints[temp].orientation[0]=Float.parseFloat(st.nextToken());
					joints[temp].orientation[1]=Float.parseFloat(st.nextToken());
					joints[temp].orientation[2]=Float.parseFloat(st.nextToken());
				}
			 }
			 i++;
			while ((data[i].trim()).equals(""))
				i++;
			st = new StringTokenizer(data[i]);
			toke = st.nextToken();
		 }
		 temp++;
	  }
	  else if(toke.equals(":bonedata"))
	  {
	     //System.out.println("in bonedata");
		 i++;
		while ((data[i].trim()).equals(""))
			i++;
		 st = new StringTokenizer(data[i]);
		 toke = st.nextToken();
		 while(!(toke.equals(":hierarchy")))
		 {
	System.out.println("in bonedata " + i + " " + linecount + " " + toke);
		    if(toke.equals("begin"))
			 {
			    i++;
				while ((data[i].trim()).equals(""))
					i++;
				st = new StringTokenizer(data[i]);
				toke = st.nextToken();
			    while (!(toke.equals("end") ))
				{
					if(toke.equals("name"))
					{
						toke = st.nextToken();
						joints[temp].name = toke;
					}
					else if(toke.equals("direction"))
					{
					    joints[temp].direction[0]=Float.parseFloat(st.nextToken());
					    joints[temp].direction[1]=Float.parseFloat(st.nextToken());
					    joints[temp].direction[2]=Float.parseFloat(st.nextToken());
					}
					else if(toke.equals("axis"))
					{
					    joints[temp].axis[0]=Float.parseFloat(st.nextToken());
					    joints[temp].axis[1]=Float.parseFloat(st.nextToken());
					    joints[temp].axis[2]=Float.parseFloat(st.nextToken());
					}
					else if(toke.equals("length"))
						joints[temp].length=Float.parseFloat(st.nextToken());
					else if(toke.equals("dof"))
					{
						while (st.hasMoreTokens()) 
						{
							toke = st.nextToken();
							if(toke.equals("tx"))
								joints[temp].dof[fr]=1;
							else if(toke.equals("ty"))
								joints[temp].dof[fr]=2;
							else if(toke.equals("tz"))
								joints[temp].dof[fr]=3;
							else if(toke.equals("rz"))
								joints[temp].dof[fr]=4;
							else if(toke.equals("ry"))
								joints[temp].dof[fr]=5;
							else if(toke.equals("rx"))
								joints[temp].dof[fr]=6;
							fr++;
						}
						joints[temp].number_dof = fr;
						fr = 0;	
					}
					i++;
					while ((data[i].trim()).equals(""))
						i++;
					st = new StringTokenizer(data[i]);
					toke = st.nextToken();
				}
				temp++;
			 }
			 i++;
			while ((data[i].trim()).equals(""))
				i++;
			 st = new StringTokenizer(data[i]);
			 toke = st.nextToken();
	     }
	  }
	  else if(toke.equals(":hierarchy"))
	  {
	     int j;
	     i = i+2;
		while ((data[i].trim()).equals(""))
			i++;
		 st = new StringTokenizer(data[i]);
		 toke = st.nextToken();
		 do{
	System.out.println("in hierarchy " + i + " " + linecount + " " + toke);

		 index = find(toke);
		 if(index == -1)
		 {
			System.out.println("something wrong in data_asf getmovinginfo");
			System.exit(0);
		 }
		 index1 = st.countTokens();
	System.out.println(index1);
		 joints[index].sons = new int[index1];
		 joints[index].number_son = index1;
         for(j=0;j<index1;j++)
		    joints[index].sons[j] = -1;
		 j=0;
		 while (st.hasMoreTokens()) 
		 {
			toke = st.nextToken();
			index1 = find(toke);
			joints[index].sons[j] = index1;
			j++;
		 }
		 i++;
	while ((data[i].trim()).equals(""))
		i++;
         st = new StringTokenizer(data[i]);
		 toke = st.nextToken();
		 }while(!(toke.equals("end")));
	  }
	  else
	  {
		i++;
	  }
	}
	System.out.println("out of reading asf");
	
     filled = true;
    }
    catch (Exception e) {
      System.out.println("Some problem in data file -- aborting");
      System.out.println(e.toString());
      clearData();
    }
  }

  // ===========================
  // findRanges -- finds mins and maxes if data wasn't preprocessed
  // ------------------------------
  private void findRanges() {
    // find the mins and maxes
    /*float x, y, z;
    minx = maxx = positions[0][0].x;
    miny = maxy = positions[0][0].y;
    minz = maxz = positions[0][0].z;
    for(int frnum = 0; frnum < numframes; ++frnum) {
      for (int mknum = 0; mknum < nummarkers; ++mknum) {
	x = positions[frnum][mknum].x;
	y = positions[frnum][mknum].y;
	z = positions[frnum][mknum].z;
	if (x <minx && x > -9.99)
	  minx = x;
	else if (x > maxx && x < 9.99)
	  maxx = x;
	if (y <miny && y > -9.99)
	  miny = y;
	else if (y > maxy && y < 9.99)
	  maxy = y;
	if (z <minz && z > -9.99)
	  minz = z;
	else if (z > maxz && z < 9.99)
	  maxz = z;
      }
    }*/
  }

  
  // ======================
  // calcDerivedData -- creates the appropriate derived data values
  // -----------------------
  public void calcDerivedData() {
    int i;
    // calculate the secs and knots
	for(i=0;i<number_joints;i++)
	{
		if(joints[i].root==1)
		{
		    System.out.println("In");
			joints[i].ab_pos[0] = joints[i].position[0];
			joints[i].ab_pos[1] = joints[i].position[1];
			joints[i].ab_pos[2] = joints[i].position[2];
			joints[i].calcasfdata(joints);
		}
	}
    for(i=0;i<2;i++)
	{
		System.out.println(joints[i].ab_pos[0]);
	    System.out.println(joints[i].ab_pos[1]);
	    System.out.println(joints[i].ab_pos[2]);
	}
  
  }

  // ======================
  // calcDerivedData -- creates the appropriate derived data values
  // -----------------------
  public void calcDerivedamcData() {
    /*knots[0] = 0.0f;
    secs = (long)((joints[0].frame_num/30.0)*1000);
    for (int j = 1; j < joints[0].frame_num; ++j) 
      knots[j] = j/(joints[0].frame_num-1);*/
    int i,j;
	float r[] = new float[3];
	int ro[]=new int[3];
	float trans[] = new float[3];
	float pos[] = new float[3];
	Matrix mat = new Matrix();
	Matrix transmat = new Matrix();
    // calculate the secs and knots
	//System.out.println("before for loop");
	for(i=0;i<number_joints;i++)
	{
		if(joints[i].root==1)
		{
		    //System.out.println("In");
			//System.out.println(joints[i].frame_num);
		    
			for(j=0;j<joints[i].frame_num;j++)
			{
			  joints[i].mat.makeIdent();
			  transmat.makeIdent();
			  for(int k=0;k<joints[i].number_dof;k++)
			  {
				if(joints[i].dof[k]==1)
					trans[0] = joints[i].dofc[j][k];
				else if(joints[i].dof[k]==2)
					trans[1] = joints[i].dofc[j][k];
				else if(joints[i].dof[k]==3)
					trans[2] = joints[i].dofc[j][k];
				else if(joints[i].dof[k]==4)
					joints[i].mat.postRotZ(joints[i].dofc[j][k]);
				else if(joints[i].dof[k]==5)
					joints[i].mat.postRotY(joints[i].dofc[j][k]);
				else if(joints[i].dof[k]==6)
					joints[i].mat.postRotX(joints[i].dofc[j][k]);
			  }
			  for(int k=0;k<3;k++)
			  {
				 joints[i].amc_position[j][k] = trans[k];
			  }
			  joints[i].calcamcdata(joints,j);
			}
			break;
		}
	}
  }

    // ===========================
    // printFileVals -- prints the values found in the file
    // ---------------------------
    public void printFileVals() {
	/*if (filled) {
	    System.out.println("\nFile values");
	    System.out.println("filename is "+setname);
	    System.out.println(numframes+" frames");
	    System.out.println(nummarkers+" markers");
	    for(int frnum = numframes-1; frnum < numframes; ++frnum) {
		System.out.println("fr "+frnum);
		  for (int mknum = nummarkers-3; mknum < nummarkers; ++mknum) 
		    System.out.println("mk "+mknum+" "+positions[frnum][mknum]);
	    }
	}
	else
	    System.out.println("Data is empty");*/
    }

    // =============================
    // printDerivedVals -- prints out the values that were derived from the data
    // ---------------------------------
    public void printDerivedVals() {
	if (filled) {
	/*    System.out.println("\nDerived Values");
	    System.out.println("filename is "+setname);
	    System.out.println(numframes+" frames");
	    System.out.println("Times from "+mintime+" to "+maxtime);
	    System.out.println("Creating "+secs+" secs for use in the alpha");
	    System.out.println("knots ...");
	    for (int frnum = numframes-1; frnum < numframes; ++frnum) {
		System.out.println(frnum+" : "+knots[frnum]);
	    }
	    System.out.println("x range is "+minx+" to "+maxx);
	    System.out.println("y range is "+miny+" to "+maxy);
	    System.out.println("z range is "+minz+" to "+maxz);*/
	}
    }

    // ======================
    // main - for testing Data
    // -----------------------
    public static void main(String args[]) throws Exception {
	/*Data d = new Data();
	String movingfile = "Data/me.txt";
	try {
	    d.setData(movingfile);
	}
	catch (Exception e) {
	    System.out.println("bad file reference .. aborting");
	    System.out.println(e);
	    System.exit(0);
	}
	if (filled) {
	    System.out.println("done filling arrays");
	    d.printFileVals();
	    d.printDerivedVals();
	}
	d.clearData();
	try {
	  d.setData("Data/vtnormJCP.txt");
	}
	catch (Exception e) {
	  System.out.println("second file problem");
	  System.out.println(e);
	  System.exit(0);
	}
	if (filled) {
	    System.out.println("done filling arrays");
	    d.printFileVals();
	    d.printDerivedVals();
	}*/
	
    }
}

