/*
 *  file:  Viewer.java
 *
 *   created by M. Geroch
 *   July 2001
 *   viewer for Motion Capture Data
 *   
 *   Modified by Wenxiang Huang
 *   viewing modified by mouse button
 *             left = rotation
 *             middle = zoom
 *             right = translation
 *   there's a reset view in the menu to restore
 *
 *  loads data from a txt file (like vicon txt) to
 *  create the marker-figure
 *
 *  allows user to start, stop animation,
 *  and reset figure to initial position
 *
 *  axis, floor and motion bounding box can all be displayed or not
 *  
 *  To Add Later:
 *      draw lines between markers
 *      show or hide individual markers
 *      
 */

import java.applet.Applet;
import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.BorderLayout;
import java.awt.event.*;

import java.lang.*;

import java.util.StringTokenizer;



import com.sun.j3d.loaders.objectfile.ObjectFile;
import com.sun.j3d.loaders.ParsingErrorException;
import com.sun.j3d.loaders.IncorrectFormatException;
import com.sun.j3d.loaders.Scene;

import java.awt.GraphicsConfiguration;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.beans.*;







//import java.util.*;

import java.applet.*;

import java.net.*;










// ********************************************
// ********************************************
public class Viewer extends Frame implements ActionListener, ItemListener{
  
  Panel p;
  MenuBar menubar;
  int height, width;
  static int frame_cur,timer_index;
  int imgx, imgy;

  int end_frame;
  static boolean ske_sel,amc_sel;
  static boolean link, hide_show;
  static boolean marker_flag, use_currentview;
  static boolean skeleton_flag;
  double degtorad;

  double factor;

  double quality;

  boolean images_flag;

  String full_filename;
  int[] hideshowselected, linkselected;
    
//  Menu file, view, envr, help, func, batch;
  Menu file, view, envr, help, func;
  CheckboxMenuItem setdefaultview, axisToggle, floorToggle, bboxToggle, transToggle, skeletonToggle, amcskeletonToggle,markeronlyToggle;
  CheckboxMenuItem hide_showtoggle, linktoggle;
  Choice combo;
  Label lL1;
  TextField field;
  ImageText imagex, imagey;
  ImageText rotx, roty, rotz, tranx, trany, tranz, scalex, scaley, scalez; 
  Scrollbar scrollbar;
  FileDialog fd;
  JFileChooser chooser;
  File filedir;
    
  BranchGroup axisRoot, floorRoot, bboxRoot, amcgroup;
  
  TransformGroup objTrans, objScale, markersTG, amcTG, axisTG;
  Transform3D startTrans,T3d,tamcd;

  Timer timer;
  int delay;

  ExampleFileFilter t3dFilter, jpgFilter, amcFilter, asfFilter, bacFilter, t2dFilter;
  FilePreviewer previewer;
  ExampleFileView fileView;
  
  Sphere marker[];

  Data thisData;
  Data_asf asf_data; 
  String jname[];
  boolean app[];
  int linknet[][];
  Marker [] markers;
  
  Appearance aquamarine3App;
  Transform3D markersT3d;
  BoundingSphere bounds;
  Alpha alpha;
  
  SimpleUniverse u;
  TimeCanvas3D c;
  List list;
  BBox bbox;
  protected PopupMenu filepopup;

  
    
    
  // ================================
  // constructors
  // --------------------------------
  Viewer() {
    width = 800;
    height = 512;
    dotheRest();
	frame_cur = 0;

	timer_index = 30;
	ske_sel = false;
	amc_sel = true;
	marker_flag = true;

	use_currentview = false;
	degtorad = 1*Math.PI/180.0;

	images_flag = false;

	factor = 0.7*0.025;
	chooser = new JFileChooser();
	filedir = new File(".");

    
	previewer = new FilePreviewer(chooser);
	chooser.setAccessory(previewer);

	// Create Custom FileView
	fileView = new ExampleFileView();
	

	//chooser.setAccessory(previewer);
	chooser.setFileView(fileView);
  }
  
  Viewer(int w, int h) {
    width = w;
    height = h;
	frame_cur = 0;

	timer_index = 30;

    jpgFilter = new ExampleFileFilter("jpg", "JPEG Compressed Image Files");
	t3dFilter = new ExampleFileFilter("t3d", "text format file");
	amcFilter = new ExampleFileFilter("amc", "Acclaim Format File");
	asfFilter = new ExampleFileFilter("asf", "Skeleton File");
	bacFilter = new ExampleFileFilter("bac", "Batch Processing");
	t2dFilter = new ExampleFileFilter("t2d", "2D export file");


	factor = 0.7*0.025;
	ske_sel = false;
	amc_sel = true;
	marker_flag = true;

	use_currentview = false;
    dotheRest();
	degtorad = 1*Math.PI/180.0;

	images_flag = false;
	chooser = new JFileChooser();
	filedir = new File(".");
	previewer = new FilePreviewer(chooser);
	chooser.setAccessory(previewer);

	// Create Custom FileView
	fileView = new ExampleFileView();
	

	//chooser.setAccessory(previewer);
	chooser.setFileView(fileView);
  }
  
  
  
  /********************************************
   * createTransInteraction
   * sets up the interactively viewable feature
   ********************************************/    
  public void createTransInteraction() {
    // sets up objTrans for interactivity
	objTrans = new TransformGroup();
    objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    objTrans.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    objTrans.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
    objTrans.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
    objTrans.setTransform(startTrans);
    
    // Create the rotate behavior node
    MyMouseRotate behavior = new MyMouseRotate();
	System.out.println("in rotate");
	behavior.xl = rotx;
	behavior.yl = roty;
    behavior.setTransformGroup(objTrans);
    objTrans.addChild(behavior);
    behavior.setSchedulingBounds(bounds);
    
    // Create the zoom behavior node
    MyMouseZoom behavior2 = new MyMouseZoom();
	behavior2.z = tranz;
    behavior2.setTransformGroup(objTrans);
    objTrans.addChild(behavior2);
    behavior2.setSchedulingBounds(bounds);
    
    // Create the translate behavior node
    MyMouseTranslate behavior3 = new MyMouseTranslate();
	behavior3.xl = tranx;
	behavior3.yl = trany;
    behavior3.setTransformGroup(objTrans);
    objTrans.addChild(behavior3);
    behavior3.setSchedulingBounds(bounds);    
  }
  
  
  public void createScaleAppear() {
    // sets up objScale so all objects appear in scene
    objScale = new TransformGroup();
    objScale.setCapability(TransformGroup.ALLOW_TRANSFORM_READ );
    objScale.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE );
    objScale.setCapability(Group.ALLOW_CHILDREN_EXTEND);
    Transform3D t3d = new Transform3D();
    objScale.setTransform(t3d);
  }
  
  /****************************************
     *  addMarkers to Figure after data is loaded
     ****************************************/
  public void addMarkers() {
    // clean up anything in currently
    int n = markersTG.numChildren();    
    for (int j = n-1; j >=0; --j)
      markersTG.removeChild(j);
    
    markers = new Marker[thisData.nummarkers];
    Point3f[] pos = new Point3f[thisData.numframes];
    for (int j=0; j < thisData.nummarkers; ++j) {
      for (int fr = 0; fr < thisData.numframes; ++fr){
	pos[fr] = thisData.positions[fr][j];
      }
      markers[j] = new Marker(0.015f, aquamarine3App, 
			      markersT3d, thisData.knots, 
			      pos, pos[0],   
			      alpha, bounds, markersTG);
    }
  }
  
  /**********************************************************
     * createFigure
     *   currently only creates the markers
     **********************************************************/
  public TransformGroup createFigure() {	
    // set up the marker colors 
    //  -- consider whether there is a better place to do this
    ColoringAttributes ca1 = new ColoringAttributes();
    ca1.setColor(ColorConstants.aquamarine3);
    aquamarine3App = new Appearance();
    aquamarine3App.setColoringAttributes(ca1);
    
    markersTG = new TransformGroup();
    markersTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    markersTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    markersTG.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    markersTG.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
    markersTG.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
    markersT3d = new Transform3D();
    markersTG.getTransform(markersT3d);
    
    // children will be added later to markersTG when a datafile is read
    // but for now, add the default ones
    addMarkers();
    
    // sets up the figure group
    TransformGroup figTrans = new TransformGroup();
    figTrans.addChild(markersTG);
    
    return figTrans;
  }
  
  
  
  /******************************************************
   * createSceneGraph
   *     the main construction for objects and behaviors
   ******************************************************/
  public BranchGroup createSceneGraph() {
    
    // initialize some of the Viewer class fields needed here
    bounds =
      new BoundingSphere(new Point3d(0.0,0.0,0.0), 1000.0);
    alpha = new Alpha(-1, thisData.secs);
    
    // Create the root of the branch graph
    BranchGroup objRoot = new BranchGroup();
    
    // Create the transform group node for interactive behavior
    
    // set up and attach auto-scale transform group
    createScaleAppear();
    objRoot.addChild(objScale);
    
    // set up and attach interactive transformation group
    createTransInteraction();
    objScale.addChild(objTrans);
    
    // Instantiate an axis and ground plane to put in the scene

	Color3f eColor    = new Color3f(0.0f, 0.0f, 0.0f);

    Color3f sColor    = new Color3f(1.0f, 1.0f, 1.0f);

    Color3f objColor  = new Color3f(0.6f, 0.6f, 0.6f);

    //Color3f lColor1   = new Color3f(1.0f, 0.0f, 0.0f);

    //Color3f lColor2   = new Color3f(0.0f, 1.0f, 0.0f);

    //Color3f alColor   = new Color3f(0.2f, 0.2f, 0.2f);

    

    

    Material m = new Material(objColor, eColor, objColor, sColor, 100.0f);

    Appearance a = new Appearance();

    m.setLightingEnable(true);

    a.setMaterial(m);

    

      
    myAxis colorAxis = new myAxis();

	Font3D f3d = new Font3D(new Font("TestFont", Font.PLAIN, 1),

			    new FontExtrusion());

	Text3D text3Dx = new Text3D(f3d, new String("X"),

			    new Point3f(5.1f, 0.0f, 0.0f));

	Text3D text3Dy = new Text3D(f3d, new String("Y"),

			    new Point3f(0.0f, 5.1f, 0.0f));

	Text3D text3Dz = new Text3D(f3d, new String("Z"),

			    new Point3f(0.0f, 0.0f, 5.1f));

	text3Dx.setCapability(Geometry.ALLOW_INTERSECT);

	text3Dy.setCapability(Geometry.ALLOW_INTERSECT);

	text3Dz.setCapability(Geometry.ALLOW_INTERSECT);

	Shape3D s3D1 = new Shape3D();

    s3D1.setGeometry(text3Dx);

    s3D1.setAppearance(a);

	Shape3D s3D2 = new Shape3D();

    s3D2.setGeometry(text3Dy);

    s3D2.setAppearance(a);

	Shape3D s3D3 = new Shape3D();

    s3D3.setGeometry(text3Dz);

    s3D3.setAppearance(a);


    axisRoot = new BranchGroup();
    axisRoot.setCapability(BranchGroup.ALLOW_DETACH);
    axisRoot.addChild(colorAxis);

	axisRoot.addChild(s3D1);

	axisRoot.addChild(s3D2);

	axisRoot.addChild(s3D3);

	T3d = new Transform3D();

	T3d.setScale(0.1);

	axisTG = new TransformGroup();

	axisTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

	axisTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

	axisTG.setCapability(TransformGroup.ALLOW_CHILDREN_READ);

	axisTG.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);

	axisTG.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);

	//axisTG.addChild(axisRoot);

	axisTG.setTransform(T3d);

	
    
    GroundPlane floor = new GroundPlane(5.0f, 0.25f);
    floorRoot = new BranchGroup();
    floorRoot.setCapability(BranchGroup.ALLOW_DETACH);
    floorRoot.addChild(floor);
	objTrans.addChild(axisTG);
    objTrans.addChild(floorRoot);

	amcgroup = new BranchGroup();
    amcgroup.setCapability(BranchGroup.ALLOW_DETACH);
	amcTG = new TransformGroup();
	amcTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	amcTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
	amcTG.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
	amcTG.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);
	amcTG.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
	tamcd = new Transform3D();
	tamcd.setScale(0.025);
	amcTG.setTransform(tamcd);
	    
	objTrans.addChild(amcTG);
        
    // Instantiate a motion bounding box
    bbox = new BBox(thisData.minx, thisData.maxx, 
		    thisData.miny, thisData.maxy,
		    thisData.minz, thisData.maxz);
    bboxRoot = new BranchGroup();
    bboxRoot.setCapability(BranchGroup.ALLOW_DETACH);
    bboxRoot.addChild(bbox);
    
    //  doesn't start out in the scene
    // objTrans.addChild(bboxRoot);
    
    // Instantiate a figure to put in the scene
    TransformGroup figTrans = createFigure();
    objTrans.addChild(figTrans);

	// Set up the background
    Color3f bgColor = new Color3f(0.1f, 0.1f, 0.1f);
    Background bgNode = new Background(bgColor);
    bgNode.setApplicationBounds(bounds);
    objRoot.addChild(bgNode);
    
    // Set up the ambient light
    Color3f ambientColor = new Color3f(0.1f, 0.1f, 0.1f);
    AmbientLight ambientLightNode = new AmbientLight(ambientColor);
    ambientLightNode.setInfluencingBounds(bounds);
    objRoot.addChild(ambientLightNode);
    
    // Set up the directional lights
    Color3f light1Color = new Color3f(1.0f, 1.0f, 0.9f);
    Vector3f light1Direction  = new Vector3f(4.0f, -7.0f, -12.0f);
    Color3f light2Color = new Color3f(0.3f, 0.3f, 0.4f);
    Vector3f light2Direction  = new Vector3f(-6.0f, -2.0f, -1.0f);
    
    DirectionalLight light1
      = new DirectionalLight(light1Color, light1Direction);
    light1.setInfluencingBounds(bounds);
    objRoot.addChild(light1);
    
    DirectionalLight light2
      = new DirectionalLight(light2Color, light2Direction);
    light2.setInfluencingBounds(bounds);
    objRoot.addChild(light2);
    
    return objRoot;
    
  }   // end of createSceneGraph
  
  
  // ====================================================
  // creating contents of viewer, both menus and canvas
  // ----------------------------------------------------
  private void createContents() {
    //  menu bar with menus
    Font f = new Font("Serif", Font.PLAIN, 12);
    setFont(f);
    menubar = new MenuBar();
    this.setMenuBar(menubar);
    
    file = new Menu("File");
    view = new Menu("View");
    envr = new Menu("Environment");
    func = new Menu("Skeleton");

//	batch = new Menu("Batch Processing");
    help = new Menu("Help");
    
    menubar.add(file);
    menubar.add(view);
    menubar.add(envr);
    menubar.add(func);

//	menubar.add(batch);
    menubar.add(help);
    menubar.setHelpMenu(help);
    
    createMenuItems(file, this,
		    new String[] { "Load text","Load asf",  "Load amc", "Export t3d", "Export t2d", "Export image", "Batch processing","Quit" },
		    new String[] { "load text","load asf",  "load amc", "export 3d text", "export 2d text", "export image", "m_both", "quit" },
		    new int[] {KeyEvent.VK_L,KeyEvent.VK_A, KeyEvent.VK_M, KeyEvent.VK_D, KeyEvent.VK_E,KeyEvent.VK_I, KeyEvent.VK_B, KeyEvent.VK_Q},
                    new int[] {1,1,0,0,0,1,1,1}
			);

/*	createMenuItems(batch,this,

	        new String[] {"both"},

			new String[] { "both"},

			null,

		    new int[] {1}

			);
*/
    createMenuItems(view, this,
		    new String[] {"reset view"},
		    new String[] {"resetv" },
		    new int[] {KeyEvent.VK_V},
                    new int[] {1} 
			);
    

	setdefaultview = new CheckboxMenuItem("set as default view",false);

	setdefaultview.addItemListener(this);

	view.add(setdefaultview);
	markeronlyToggle = new CheckboxMenuItem("Markers",true);
	markeronlyToggle.addItemListener(this);
	envr.add(markeronlyToggle);
	skeletonToggle = new CheckboxMenuItem("Stick Figure", false);
    skeletonToggle.addItemListener(this);
	envr.add(skeletonToggle);
	amcskeletonToggle = new CheckboxMenuItem("3D Skeleton",true);
	amcskeletonToggle.addItemListener(this);
    envr.add(amcskeletonToggle);
    axisToggle = new CheckboxMenuItem("Axis", false);
    axisToggle.addItemListener(this);
    envr.add(axisToggle);
    floorToggle = new CheckboxMenuItem("Floor", true);
    floorToggle.addItemListener(this);
    envr.add(floorToggle);
    
	linktoggle = new CheckboxMenuItem("link two joints",false);
	linktoggle.addItemListener(this);
	link=false;
	hide_show = true;
	func.add(linktoggle);
	hide_showtoggle = new CheckboxMenuItem("hide or show joints",true);
	hide_showtoggle.addItemListener(this);
	func.add(hide_showtoggle);

	timer = new Timer(33,this);
	timer.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	anit();}
    } );

    createMenuItems(help, this,
		    new String[] { "About"}, new String[] {"about"},
		    new int[] {KeyEvent.VK_A},
                    new int[] {1}
		    );
    
    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) { System.exit(0); }
    });
    
    // ---- set up the frame's layout and 3D canvas
    setBackground(new Color(0.7f, 0.8f, 0.9f));
    setLayout(new BorderLayout());
    GraphicsConfiguration config =
      SimpleUniverse.getPreferredConfiguration();
    c = new TimeCanvas3D(config);

	FieldValidator validator = new FieldValidator();
    FieldValidatord validatord = new FieldValidatord();
    
    Panel pL1 = new Panel(new GridLayout(1,1));
    lL1 = new Label("subject file");

    Panel pL = new Panel(new GridLayout(11,2));
    Label label1 = new Label("Rotate        X");
    label1.setAlignment(Label.RIGHT);
	rotx = new ImageText("0.0",7);
	Label label2 = new Label("                   Y");
    label2.setAlignment(Label.RIGHT);
    roty = new ImageText("0.0",7);
    Label label3 = new Label("                   Z");
    label3.setAlignment(Label.RIGHT);
    rotz = new ImageText("0.0",7);
	System.out.println("after rotz");
    Label label4 = new Label("Translate    X");
    label4.setAlignment(Label.RIGHT);
    tranx = new ImageText("0.0",7);

	tranx.val = 0.0;
    Label label5 = new Label("                   Y");
    label5.setAlignment(Label.RIGHT);
    trany = new ImageText("-0.7",7);
	trany.val = -0.7/factor;

	System.out.println(trany.val);

	trany.setText(Double.toString(trany.val));


    Label label6 = new Label("                   Z");
    label6.setAlignment(Label.RIGHT);
    tranz = new ImageText("-0.7",7);

	System.out.println(factor);
	tranz.val = -0.7/factor;

	System.out.println(tranz.val);

	tranz.setText(Double.toString(tranz.val));
    Label label7 = new Label("Scale          X");
    label7.setAlignment(Label.RIGHT);
//    scalex = new ImageText("1.0",7);
//    scalex.val = 1.0;
//    Label label8 = new Label("                   Y");
//    label8.setAlignment(Label.RIGHT);
//    scaley = new ImageText("1.0",7);
//    scaley.val = 1.0;
//    Label label9 = new Label("                   Z");
//    label9.setAlignment(Label.RIGHT);
//    scalez = new ImageText("1.0",7);
//    scalez.val = 1.0;
    Label label10 = new Label("Image Size X");
    label10.setAlignment(Label.RIGHT);
    imagex = new ImageText("640",1);
	c.imagex = imagex;
    Label label11 = new Label("                   Y");
    label11.setAlignment(Label.RIGHT);
    imagey = new ImageText("480",1);
	c.imagey = imagey;

    rotx.addKeyListener(validatord);
    roty.addKeyListener(validatord);
    rotz.addKeyListener(validatord);
    tranx.addKeyListener(validatord);
    trany.addKeyListener(validatord);
    tranz.addKeyListener(validatord);
//    scalex.addKeyListener(validatord);
//    scaley.addKeyListener(validatord);
//    scalez.addKeyListener(validatord);
    imagex.addKeyListener(validator);
    imagey.addKeyListener(validator);

    rotx.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	if(rotx.getText().equals("."));
	else
	{
	double degree = Double.parseDouble(rotx.getText());
	degree = degree - rotx.val;
	System.out.println(degree);
    rotx.val += degree;
	Transform3D rot = new Transform3D();
	rot.rotX(degree*degtorad);
    Transform3D tran = new Transform3D();
	objTrans.getTransform(tran);
	tran.mul(rot);
	objTrans.setTransform(tran);
	}
      }
    } );

    roty.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	 if(roty.getText().equals("."));
	 else
	 { 
	  double degree = Double.parseDouble(roty.getText());
	  degree = degree - roty.val;
	  System.out.println(degree);
	  roty.val += degree;
	  roty.setText(Double.toString(roty.val));
	  Transform3D rot = new Transform3D();
	  rot.rotY(degree*degtorad);
      Transform3D tran = new Transform3D();
	  objTrans.getTransform(tran);
	  tran.mul(rot);
	  objTrans.setTransform(tran);
	 }
      }
    } );

    rotz.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	  if(rotz.getText().equals("."));
	else
	{
	  double degree = Double.parseDouble(rotz.getText());
	  degree = degree - rotz.val;
	  rotz.val += degree;
	  Transform3D rot = new Transform3D();
	  rot.rotZ(degree*degtorad);
      Transform3D tran = new Transform3D();
	  objTrans.getTransform(tran);
	  tran.mul(rot);
	  objTrans.setTransform(tran);
	}
      }
    } );

	tranz.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	  if(tranz.getText().equals("."));
	  else if(tranz.getText().equals("-"));
	else
	{
	  double degree = Double.parseDouble(tranz.getText());
	  degree = degree - tranz.val;
	  tranz.val += degree;
	  Transform3D tran = new Transform3D();
	  tran.set(new Vector3f(0.0f, 0.0f, (float)degree*(float)factor));
      Transform3D tt = new Transform3D();
	  objTrans.getTransform(tt);
	  tt.mul(tran,tt);
	  objTrans.setTransform(tt);

	  System.out.println(tt);
	  
	}
      }
    } );

	tranx.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	  if(tranx.getText().equals("."));
	  else if(tranx.getText().equals("-"));
	else
	{
	  double degree = Double.parseDouble(tranx.getText());
	  degree = degree - tranx.val;
	  tranx.val += degree;
	  Transform3D tran = new Transform3D();
	  tran.set(new Vector3f((float)degree*(float)factor, 0.0f, 0.0f));
      Transform3D tt = new Transform3D();
	  objTrans.getTransform(tt);
	  tt.mul(tran,tt);
	  objTrans.setTransform(tt);

	  System.out.println(tt);


	}
      }
    } );

	trany.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	  if(trany.getText().equals("."));
	  else if(trany.getText().equals("-"));
	else
	{
	  double degree = Double.parseDouble(trany.getText());
	  degree = degree - trany.val;
	  trany.val += degree;
	  Transform3D tran = new Transform3D();
	  tran.set(new Vector3f(0.0f, (float)degree*(float)factor, 0.0f));
      Transform3D tt = new Transform3D();
	  objTrans.getTransform(tt);
	  tt.mul(tran,tt);
	  objTrans.setTransform(tt);
	}
      }
    } );

    imagex.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	  
	  int tempx = Integer.parseInt(imagex.getText());
	  int diff = tempx-imagex.value;
	  imagex.value=tempx;
	  width = width + diff;
	  setSize(width,height);
	  repaint();
	  show();
      }
    } );

    imagey.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	  int tempy = Integer.parseInt(imagey.getText());
	  int diff = tempy-imagey.value;
	  imagey.value=tempy;
	  height = height + diff;
	  setSize(width,height);
	  repaint();
	  show();
      }
    } );



    pL1.add(lL1);
    pL.add(label1);
    pL.add(rotx);
    pL.add(label2);
    pL.add(roty);
    pL.add(label3);
    pL.add(rotz);
    pL.add(label4);
    pL.add(tranx);
    pL.add(label5);
    pL.add(trany);
    pL.add(label6);
    pL.add(tranz);
//    pL.add(label7);
//    pL.add(scalex);
//    pL.add(label8);
//    pL.add(scaley);
//    pL.add(label9);
//    pL.add(scalez);
    pL.add(label10);
    pL.add(imagex);
    pL.add(label11);
    pL.add(imagey);

    Panel pL6 = new Panel(new BorderLayout());
    pL6.add("North",pL1);
    pL6.add("Center",pL);

    
    //Panel pL = new Panel(new GridLayout(10,1));
    //lL1 = new Label("subject file");
	//pL.add(lL1);

	    Panel pL2 = new Panel(new GridLayout(1,10));	

    Button bL1 = new Button(" > ");
    Button bL2 = new Button(" [] ");
    Button bL3 = new Button(" >| ");
    Button bL4 = new Button(" >|| ");
    Button bL5 = new Button(" |< ");    
    Button bL6 = new Button(" ||< ");
    Label label = new Label("Frame:");
    label.setAlignment(Label.RIGHT);
    field = new TextField("1",5);

    combo = new Choice();
    combo.add("30 fps");
    combo.add("60 fps");
    combo.add("90 fps");
    combo.add("120 fps");

    

    //FieldValidator validator = new FieldValidator();
    field.addKeyListener(validator);

     field.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	//change this to go to a particular frame in animation
	//enableAnim();
    frame_cur = Integer.parseInt(field.getText());
	//hideamcskeleton();
	showamc(marker_flag,ske_sel,amc_sel);}
    } );
    
    combo.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
	// change this to playback animation at new rate
	//disableAnim();
	if(combo.getSelectedIndex()==0)
	{
		timer.setDelay(1000/30);
		timer_index = 30;
	}
	else if(combo.getSelectedIndex()==1)
	{
		timer.setDelay(1000/60);
		timer_index = 60;
	}
	else if(combo.getSelectedIndex()==2)
	{
		timer.setDelay(1000/90);
		timer_index = 90;
	}
	else if(combo.getSelectedIndex()==3)
	{
		timer.setDelay(1000/120);
		timer_index = 120;
	}
	//enableAnim();
	}
    } );
    
  
 
    bL1.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	enableAnim();}
    } );
    
    bL2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	disableAnim();}
    } );

    bL3.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	disableAnim();
	if(asf_data.amcfilled)
	{
	frame_cur++;
	if(frame_cur > (asf_data.frame_number-1))frame_cur=asf_data.frame_number-1;
	scrollbar.setValue(frame_cur);
	field.setText(Integer.toString(frame_cur));
	//hideamcskeleton();
	showamc(marker_flag,ske_sel,amc_sel);
	}
	}
    } );

    bL4.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	disableAnim();
	frame_cur=asf_data.frame_number-1;
	scrollbar.setValue(frame_cur);
	field.setText(Integer.toString(frame_cur));
	//hideamcskeleton();
	showamc(marker_flag,ske_sel,amc_sel);}
    } );

    bL5.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	disableAnim();
	frame_cur--;
	if(frame_cur < 0)frame_cur=0;
	scrollbar.setValue(frame_cur);
	field.setText(Integer.toString(frame_cur));
	//hideamcskeleton();
	showamc(marker_flag,ske_sel,amc_sel);}
    } );

    bL6.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        disableAnim();
		frame_cur=0;
		scrollbar.setValue(frame_cur);
		field.setText(Integer.toString(frame_cur));
		//hideamcskeleton();
		showamc(marker_flag,ske_sel,amc_sel);
		}
    } );



    pL2.add(bL6);
    pL2.add(bL5);
    pL2.add(bL1);
    pL2.add(bL2);
    pL2.add(bL3);
    pL2.add(bL4);
    pL2.add(label);
    pL2.add(field);
    pL2.add(combo);

    list = new List();
    list.setMultipleMode(true);
    ListListener listen = new ListListener();
    list.addItemListener(listen);
    linkselected = list.getSelectedIndexes();

    hideshowselected = list.getSelectedIndexes();
	Panel pL4 = new Panel(new GridLayout(1,10));
	scrollbar = new Scrollbar(Scrollbar.HORIZONTAL,
                              0, //initialvalue
                              1, //visible
                              0, //minimum
                              500 //maximum
                              );
    
    scrollbar.setUnitIncrement(1);
    scrollbar.setBlockIncrement(20);

    scrollbar.addAdjustmentListener(new AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent e) {
        int value = scrollbar.getValue();
        field.setText(Integer.toString(value));
		frame_cur = value;
		//hideamcskeleton();
		System.out.println("in linstern");
		showamc(marker_flag,ske_sel,amc_sel);
      }
    });
    pL4.add(scrollbar);
	Panel pL5 = new Panel(new GridLayout(1,2));
	pL5.add(pL2);
	pL5.add(pL4);
	add("Center", c);
    add("West", pL6);
	add("East", list);
    this.add("South",pL5);    
  
    setSize(this.width, this.height);
  }    // end of createContents


  public void manage(int number_joints,int [] selected)
  {

    System.out.println("selected[0]");

    System.out.println(selected[0]);

	System.out.println("selected[1]");

	System.out.println(selected[1]);
	int i,j;
	for(i=0;i<number_joints-1;i++)
	{
		if(linknet[selected[0]][i]==selected[1])
		{
			linknet[selected[0]][i]=-1;
			return;
		}
	}
	for(i=0;i<number_joints-1;i++)
	{
		if(linknet[selected[0]][i]==-1)
		{
			linknet[selected[0]][i]=selected[1];
			return;
		}
	}
	
  }


  public class ListListener implements ItemListener {
    public void itemStateChanged(ItemEvent e){
    if(hide_show)
		System.out.println("hide_show:true");
    else
		System.out.println("hide_show:false");
    if(link)
		System.out.println("link:true");
    else
		System.out.println("link:false");
    
    List thelist=(List)e.getSource();
     
    if((thelist.getSelectedIndexes()).length==0){
		for(int i=0; i<app.length;i++){
			app[i]=true;
		}
		showamc(marker_flag,ske_sel,amc_sel);
		return;
    }
    else
	{
	    int [] selected = thelist.getSelectedIndexes();
		if(hide_show)
	    {
	      for(int i=0; i<app.length;i++)
				app[i]=true;

	      for (int i=0; i<selected.length; i++){
				app[selected[i]]=false;
	      }
	      
		  showamc(marker_flag,ske_sel,amc_sel);
	      link = false;
	    }
		else if(link)
	    {
	      if (selected.length==2)
			{
				manage(asf_data.number_joints,selected);
				//hideamcskeleton();
				showamc(marker_flag,ske_sel,amc_sel);
				thelist.deselect(selected[0]);
				thelist.deselect(selected[1]);
			}		
	      hide_show=false;
	    }
	}
    }
    
  }


  
  // ======================================================
  // convenience routine for adding menu items to menu pane 
  // ----------------------------------------------------
  protected static void createMenuItems(Menu pane, ActionListener listener,
					String[] labels, String[] commands,
					int[] shortcuts,int[] enable) {
    for(int i = 0; i < labels.length; i++) {
      MenuItem mi = new MenuItem(labels[i]);
      mi.addActionListener(listener);
      if ((commands != null) && (commands[i] != null))
	mi.setActionCommand(commands[i]);
      if ((shortcuts != null) && (shortcuts[i] != 0))
	mi.setShortcut(new MenuShortcut(shortcuts[i]));
      if ((enable != null) && (enable[i] == 0))
        mi.setEnabled(false);
      pane.add(mi);
    }
  }  // end of createMenuItems()
  
    // Validates text field to allow only numeric entry on the fly
   class FieldValidator extends KeyAdapter {
    public void keyPressed(KeyEvent event) {
	TextField field = (TextField)event.getSource();
	String oldstring = field.getText();
	String newstring = new String();
	int value = 0, newValue;

	if(event.getKeyCode()==KeyEvent.VK_TAB || event.getKeyCode()==KeyEvent.VK_BACK_SPACE || event.getKeyCode()==KeyEvent.VK_DELETE)

		;	

	else{

	  try {

		if (!oldstring.equals(""))
		  value = Integer.parseInt(oldstring);

		newstring += event.getKeyChar();
		newValue = Integer.parseInt(newstring);
		}
	     catch(NumberFormatException e) {
		event.consume();
		field.selectAll();
	     }

	    }
         }
      }
	class FieldValidatord extends KeyAdapter {
    public void keyPressed(KeyEvent event) {
	TextField field = (TextField)event.getSource();
	String oldstring = field.getText();
	String newstring = new String();
	double value = 0, newValue;


	if(event.getKeyCode()==KeyEvent.VK_TAB || event.getKeyCode()==KeyEvent.VK_BACK_SPACE || event.getKeyCode()==KeyEvent.VK_DELETE)
		;

	else{

	  try {

		if (!oldstring.equals("")&&!oldstring.equals(".")&&!oldstring.equals("-")&&!oldstring.equals("-."))
		  value = Double.parseDouble(oldstring);

		newstring += event.getKeyChar();
		if(newstring.equals("."));
		else if(newstring.equals("-"));
		else
		  newValue = Integer.parseInt(newstring);
		}
	     catch(NumberFormatException e) {
		 event.consume();
		 field.selectAll();
	     }

          }
         }
      }
  
  
  
  // =======================================================
  // responses to user interaction
  // ------------------------------------------------
  // ------------ make popup menus available for menubar commands
  public void processMouseEvent(MouseEvent e) {
    if ((filepopup!= null) && e.isPopupTrigger())
      filepopup.show(this, e.getX(), e.getY());
    else
      super.processMouseEvent(e);
  } // end of processMouseEvent()
  
  
    // ------------- response to selecting the load file command
  private void loadFile(){
    
	chooser.addChoosableFileFilter(t3dFilter);
	//System.out.println(chooser);
	if(filedir == null)filedir = new File(".");
	chooser.setCurrentDirectory(filedir);
	chooser.showOpenDialog(null);
	filedir = chooser.getSelectedFile();
	
	if(filedir == null)return;
	
    String fullfilename = filedir.getAbsolutePath();
	chooser.removeChoosableFileFilter(t3dFilter);
	System.out.println(fullfilename);
    disableAnim();
    thisData.clearData();
    hideBbox();
    try {
      thisData.setData(fullfilename);
     }
    catch (Exception e) {
      System.out.println("loadFile: bad file reference .. aborting");
      System.out.println(e);
      //System.exit(0);
    }
    alpha = new Alpha(-1, thisData.secs);
    lL1.setText(thisData.setname);
    try {
      addMarkers();
      bbox.newSize(thisData.minx, thisData.maxx, 
		   thisData.miny, thisData.maxy,
		   thisData.minz, thisData.maxz);
    }
    catch (Exception e) {
      System.out.println("problem adding new markers .. aborting");
      System.out.println(e);
      //System.exit(0);
    }
  }

  public void anit()
  {
  if(asf_data.amcfilled)
	{
	    int i;

		if(images_flag)

		{

			int dotindex = full_filename.lastIndexOf('.');

			String setname = full_filename.substring(0, dotindex);

			if(asf_data.joints[0].frame_num>=100)

			{

				if(frame_cur<10)

					setname = setname + "00"+Integer.toString(frame_cur) + ".jpg";

				else if(frame_cur<100)

					setname = setname + "0"+Integer.toString(frame_cur) + ".jpg";

				else

					setname = setname + Integer.toString(frame_cur) + ".jpg";	

			}

			else if(asf_data.joints[0].frame_num>=10 && asf_data.joints[0].frame_num<100)

			{
				if(frame_cur<10)
					setname = setname + "0"+Integer.toString(frame_cur) + ".jpg";
				else
					setname = setname + Integer.toString(frame_cur) + ".jpg";	
			}
			else
				setname = setname + Integer.toString(frame_cur) + ".jpg";
			c.saveFrame(setname,quality);
			if(frame_cur==end_frame)
			{
			 images_flag = false;
			 timer.setDelay(1000/timer_index);
			 disableAnim();
			 return;
			}
		}
		
	    frame_cur++;
		scrollbar.setValue(frame_cur);
		field.setText(Integer.toString(frame_cur));
		if(frame_cur>=asf_data.joints[0].frame_num)frame_cur = 0;
		showamc(marker_flag,ske_sel,amc_sel);
	}
	else
	{
    for (int j=0; j < thisData.nummarkers; ++j) {
      markers[j].starter();	}
	}
 }

  public void showamc(boolean dot,boolean ske_sel,boolean amc_sel)
  {
	hideamcskeleton();
    if(asf_data.amcfilled==false)return;
    int nc = amcgroup.numChildren();
    for (int j=nc-1; j >= 0; j--)  
		amcgroup.removeChild(j);
	if(dot)
	{
		jointslon ske1 = new jointslon(asf_data.joints,asf_data.number_joints,frame_cur,app);
		amcgroup.addChild(ske1);
	}	
	if(ske_sel)
	{
		skeleton_s ske2 = new skeleton_s(asf_data.joints,asf_data.number_joints,frame_cur,app,linknet);
		amcgroup.addChild(ske2);
	}
	if(amc_sel)
	{
		skeleton ske3 = new skeleton(asf_data.joints,asf_data.number_joints,frame_cur,app,linknet);
		amcgroup.addChild(ske3);
	}
	amcTG.addChild(amcgroup);
  }

  private void loadFile_amc(){

	chooser.addChoosableFileFilter(amcFilter);
	if(filedir == null)filedir = new File(".");
	chooser.setCurrentDirectory(filedir);
	chooser.showOpenDialog(null);
	filedir = chooser.getSelectedFile();
	
	if(filedir == null)return;
	
    String fullfilename = filedir.getAbsolutePath();
	chooser.removeChoosableFileFilter(amcFilter);
	//System.out.println(fullfilename);
	String filename = filedir.getName();
	
    disableAnim();
	asf_data.clearamcData();
    hideBbox();
    try {
      asf_data.setamcData(fullfilename,filename);
     }
    catch (Exception e) {
      System.out.println("loadFile: bad file reference .. aborting");
      System.out.println(e);
    }
    alpha = new Alpha(-1, asf_data.secs);
    lL1.setText(asf_data.amcname);
  }

  private void loadFile_asf(){

	asf_data.filled = false;
	chooser.addChoosableFileFilter(asfFilter);
	
	if(filedir == null)filedir = new File(".");
	chooser.setCurrentDirectory(filedir);
	chooser.showOpenDialog(null);
	filedir = chooser.getSelectedFile();
	
	if(filedir == null)return;
	
    String fullfilename = filedir.getAbsolutePath();
	chooser.removeChoosableFileFilter(asfFilter);
	System.out.println(fullfilename);
	String filename = filedir.getName();

	disableAnim();
	asf_data.clearData();

System.out.println("d");
    hideBbox();

	System.out.println("e");


    try {

	System.out.println("f");


      asf_data.setData(fullfilename,filename);
     }
    catch (Exception e) {
      System.out.println("loadFile: bad file reference .. aborting");
      System.out.println(e);
    }

	System.out.println("g");


    lL1.setText(asf_data.setname);
  }

private void loadFile_hrc(){

	asf_data.filled = false;
	chooser.addChoosableFileFilter(t3dFilter);
	
	if(filedir == null)filedir = new File(".");
	chooser.setCurrentDirectory(filedir);
	chooser.showOpenDialog(null);
	filedir = chooser.getSelectedFile();
	
	if(filedir == null)return;
	
    String fullfilename = filedir.getAbsolutePath();
	chooser.removeChoosableFileFilter(t3dFilter);
	System.out.println(fullfilename);
	String filename = filedir.getName();

	disableAnim();
	asf_data.clearData();

	hideBbox();
    try {
     asf_data.sethrcData(fullfilename,filename);
     }
    catch (Exception e) {
      System.out.println("loadFile: bad file reference .. aborting");
      System.out.println(e);
    }

	System.out.println("g");


    lL1.setText(asf_data.setname);
  }


  // -------------- response to button to start animation
  public void enableAnim() {
    // call the various marker classes to enable their
    // interpolators
	if(asf_data.amcfilled)timer.start();
	for (int j=0; j < thisData.nummarkers; ++j) {
      markers[j].starter();	}
  }
  
  // ------------- response to button to stop animation
  public void disableAnim() {
    // call the various marker classes to disable their
    // interpolators
	timer.stop();
    for (int j=0; j < thisData.nummarkers; ++j) {
      markers[j].stopper();	}
  }
  
  // ----------- response to menu selection reset Figure
  public void resetFigure() {
    for (int j=0; j < thisData.nummarkers; ++j) {
      markers[j].setLoc(-1);
    }    
  }

  public void hideamcskeleton() {
    if(asf_data.amcfilled)
		amcTG.removeChild(0);
    }

  /*public void hideskeleton() {
    int nc = skeletonRoot.numChildren();
    Node nd;
    int t = -1;
    for (int j=0; j < nc; ++j) { 
      nd = skeletonRoot.getChild(j);
      if (nd.equals((Node)skeletonb)) {
	t = j; } }
    if (t >= 0) {
      skeletonRoot.removeChild(t);
	  }
    }*/
  // ----------- response to axisToggle (menu) deselection
  public void hideAxis() {
    /*int nc = objTrans.numChildren();
    Node nd;
    int t = -1;
    for (int j=0; j < nc; ++j) { 
      nd = objTrans.getChild(j);
      if (nd.equals((Node)axisTG)) {
	t = j; } }
    if (t >= 0) {
      objTrans.removeChild(t);}*/

	  axisTG.removeChild(0);
    }
  
  // ------------ response to floorToggle (menu) deselection
  public void hideFloor() {
    int nc = objTrans.numChildren();
    Node nd;
    int t = -1;
    for (int j=0; j < nc; ++j) { 
      nd = objTrans.getChild(j);
      if (nd.equals((Node)floorRoot)) {
		t = j;}}
    if (t >= 0) {
	    objTrans.removeChild(t); }
  }   
    
  // -------------- response to bboxToggle (menu) deselection
  public void hideBbox() {
    int nc = objTrans.numChildren();
    Node nd;
    int t = -1;
    for (int j=0; j < nc; ++j) { 
      nd = objTrans.getChild(j);
      if (nd.equals((Node)bboxRoot)) {
	t = j; } }
    if (t >= 0) {
      objTrans.removeChild(t);}    
  }
  
  
  
  // =============================================
  // method drivers for actionlistener and itemlistener 
  //   interface  for menubar commands
  // ----------------------------------------------
  // ---------------
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    MenuItem temp;
	//System.out.println(e.toString());
	if(cmd==null)
	  {
	    //System.out.println("anit");
		//anit();
		return;
	  }

	
	if (cmd.equals("quit"))  
      System.exit(0);
    else if (cmd.equals("load text"))
	{
	  if(asf_data.filled)
	     list.removeAll();
   	  if(asf_data.amcfilled)
	  {
		hideamcskeleton();
		int nc = amcgroup.numChildren();
		for (int j=nc-1; j >= 0; j--)  
			amcgroup.removeChild(j);
	  }
      loadFile_hrc();
	  System.out.println("before go geometry");
	  if(asf_data.filled)
		{
		app = new boolean[asf_data.number_joints];
		linknet = new int[asf_data.number_joints][asf_data.number_joints-1];
		for(int i=0;i<asf_data.number_joints;i++)
		{
			for(int j=0;j<asf_data.number_joints-1;j++)
				linknet[i][j]=-1;
			app[i] = true;
		}
		for(int i=0;i<asf_data.number_joints;i++)
		{
			for(int j=0;j<asf_data.joints[i].number_son;j++)
			{
				linknet[i][j]=asf_data.joints[i].sons[j];
			}
		}
		jname = new String[asf_data.number_joints];
		for(int i=0;i<asf_data.number_joints;i++)
		    {
		      jname[i] = asf_data.joints[i].name;
		      list.add(jname[i]);
		    }
		temp = file.getItem(2);
        temp.setEnabled(false);
		}
	  if(asf_data.amcfilled)
		{
		if(!use_currentview)
			objTrans.setTransform(startTrans);
		scrollbar.setMaximum(asf_data.frame_number);
		frame_cur=0;
		scrollbar.setValue(frame_cur);
		field.setText(Integer.toString(frame_cur));

		amcskeletonToggle.setState(false);
		amcskeletonToggle.setEnabled(false);
		skeletonToggle.setState(true);
		amc_sel = false;
		marker_flag=true;
		ske_sel = true;
		
        if(!use_currentview)
		{
		Transform3D tr = new Transform3D();
		c.getVworldToImagePlate(tr);
		Transform3D tri = new Transform3D(tr);
		tri.invert();
		int width = c.getSize().width;
		int height = c.getSize().height;
		width /= 2;
		height /= 2;
		Point3d imagePlatePoint = new Point3d();
		c.getPixelLocationInImagePlate(width,
                                         height,
                                         imagePlatePoint);
		tri.transform(imagePlatePoint);
        Transform3D tt = new Transform3D();
		amcTG.getTransform(tt);
		Transform3D ttt = new Transform3D();
		objTrans.getTransform(ttt);
        ttt.mul(tt);
		Point3d point = new Point3d(asf_data.joints[0].amc_position[frame_cur][0],asf_data.joints[0].amc_position[frame_cur][1],asf_data.joints[0].amc_position[frame_cur][2]);
		ttt.transform(point);
        objTrans.getTransform(ttt);
		Transform3D tran = new Transform3D();
		double x,y,z;
		x = imagePlatePoint.x-point.x;
		y = imagePlatePoint.y-point.y;
		z = imagePlatePoint.z-point.z;
	    tran.set(new Vector3d(x,y,z));
		tran.mul(ttt);
		objTrans.setTransform(tran);
		Point3d point1 = new Point3d(0.0,0.0,0.0);
		tran.transform(point1);
		tranx.val=point1.x/(0.7*0.025);
		tranx.setText(Double.toString(tranx.val));
		trany.val=point1.y/(0.7*0.025);
		trany.setText(Double.toString(trany.val));
		tranz.val=point1.z/(0.7*0.025);
		tranz.setText(Double.toString(tranz.val));
        }
		jointslon ske1 = new jointslon(asf_data.joints,asf_data.number_joints,frame_cur,app);
		amcgroup.addChild(ske1);
		skeleton ske2 = new skeleton(asf_data.joints,asf_data.number_joints,frame_cur,app,linknet);
		amcgroup.addChild(ske2);
		amcTG.addChild(amcgroup);
	  }

      temp = file.getItem(3);
      temp.setEnabled(true);
      temp = file.getItem(4);
      temp.setEnabled(true);
      temp = file.getItem(5);
      temp.setEnabled(true);          
		
	}
      //loadFile();
	else if (cmd.equals("load asf"))
	{

	  if(asf_data.filled)
	     list.removeAll();
	  if(asf_data.amcfilled)
	  {
		hideamcskeleton();
		int nc = amcgroup.numChildren();
		for (int j=nc-1; j >= 0; j--)  
			amcgroup.removeChild(j);
		asf_data.clearamcData();
	  }
      loadFile_asf();
	  System.out.println("before go geometry");
	  if(asf_data.filled)
		{
		app = new boolean[asf_data.number_joints];
		linknet = new int[asf_data.number_joints][asf_data.number_joints-1];
		for(int i=0;i<asf_data.number_joints;i++)
		{
			for(int j=0;j<asf_data.number_joints-1;j++)
				linknet[i][j]=-1;
			app[i] = true;
		}

		for(int i=0;i<asf_data.number_joints;i++)

		{

			for(int j=0;j<asf_data.joints[i].number_son;j++)

			{

				linknet[i][j]=asf_data.joints[i].sons[j];

			}

		}

		jname = new String[asf_data.number_joints];
		for(int i=0;i<asf_data.number_joints;i++)
		    {
		      jname[i] = asf_data.joints[i].name;
		      list.add(jname[i]);
		    }

		temp = file.getItem(2);

        temp.setEnabled(true);
		}
          


	}
	else if (cmd.equals("load amc"))
	{

	  if(asf_data.amcfilled)
	  {
		hideamcskeleton();
		int nc = amcgroup.numChildren();
		for (int j=nc-1; j >= 0; j--)  
			amcgroup.removeChild(j);
	  }
      loadFile_amc();
	  if(asf_data.amcfilled)
		{

		if(!use_currentview)
			objTrans.setTransform(startTrans);
		scrollbar.setMaximum(asf_data.frame_number);
		frame_cur=0;
		scrollbar.setValue(frame_cur);
		field.setText(Integer.toString(frame_cur));

		amcskeletonToggle.setEnabled(true);
//		amcskeletonToggle.setState(true);
		
/*		amc_sel = true;
		marker_flag=true;
		ske_sel = true;*/
		
        if(!use_currentview)
		{
		Transform3D tr = new Transform3D();
		c.getVworldToImagePlate(tr);
		Transform3D tri = new Transform3D(tr);
		tri.invert();
		int width = c.getSize().width;
		int height = c.getSize().height;
		width /= 2;
		height /= 2;
		Point3d imagePlatePoint = new Point3d();
		c.getPixelLocationInImagePlate(width,
                                         height,
                                         imagePlatePoint);
		tri.transform(imagePlatePoint);
        Transform3D tt = new Transform3D();
		amcTG.getTransform(tt);
		Transform3D ttt = new Transform3D();
		objTrans.getTransform(ttt);
        ttt.mul(tt);
		Point3d point = new Point3d(asf_data.joints[0].amc_position[frame_cur][0],asf_data.joints[0].amc_position[frame_cur][1],asf_data.joints[0].amc_position[frame_cur][2]);
		ttt.transform(point);
        objTrans.getTransform(ttt);
		Transform3D tran = new Transform3D();
		double x,y,z;
		x = imagePlatePoint.x-point.x;
		y = imagePlatePoint.y-point.y;
		z = imagePlatePoint.z-point.z;
	    tran.set(new Vector3d(x,y,z));
		tran.mul(ttt);
		objTrans.setTransform(tran);
		Point3d point1 = new Point3d(0.0,0.0,0.0);
		tran.transform(point1);
		tranx.val=point1.x/(0.7*0.025);
		tranx.setText(Double.toString(tranx.val));
		trany.val=point1.y/(0.7*0.025);
		trany.setText(Double.toString(trany.val));
		tranz.val=point1.z/(0.7*0.025);
		tranz.setText(Double.toString(tranz.val));
        }
		if(marker_flag)
		{
			jointslon ske1 = new jointslon(asf_data.joints,asf_data.number_joints,frame_cur,app);
			amcgroup.addChild(ske1);
		}
		if(ske_sel)
		{
			skeleton_s ske2 = new skeleton_s(asf_data.joints,asf_data.number_joints,frame_cur,app,linknet);
			amcgroup.addChild(ske2);
		}
		if(amc_sel)
		{
			skeleton ske2 = new skeleton(asf_data.joints,asf_data.number_joints,frame_cur,app,linknet);
			amcgroup.addChild(ske2);
		}
		amcTG.addChild(amcgroup);
	  }

          temp = file.getItem(3);
          temp.setEnabled(true);
          temp = file.getItem(4);
          temp.setEnabled(true);
          temp = file.getItem(5);
          temp.setEnabled(true);

	}
	else if (cmd.equals("export 3d text"))
	  {
		/*FileDialog fd = new FileDialog(this, "ExportFile", FileDialog.SAVE);
		fd.setDirectory(filedir);

		fd.setFile("*.t3d");
		fd.show();
		filedir = fd.getDirectory();*/

		chooser.addChoosableFileFilter(t3dFilter);
		if(filedir == null)filedir = new File(".");
		chooser.setCurrentDirectory(filedir);
		chooser.showSaveDialog(null);
		filedir = chooser.getSelectedFile();
		
		if(filedir == null)return;
		
		String fullfilename = filedir.getAbsolutePath();
		chooser.removeChoosableFileFilter(t3dFilter);
		System.out.println(fullfilename);

		String filename = filedir.getName();
		if (filename == null) 
		  return;
		int dotindex = filename.lastIndexOf('.');
		if(dotindex == -1)
			fullfilename = fullfilename+".t3d";
		else
		{
			String fn;
		  	fn = filename.substring(dotindex+1);
			fn.toLowerCase();
			System.out.println(fn);
			if(!fn.equals("t3d"))fullfilename=fullfilename+".t3d";
		}

		disableAnim();

		Transform3D tt = new Transform3D();

		amcTG.getTransform(tt);

		Transform3D ttt = new Transform3D();

		objTrans.getTransform(ttt);

        tt.mul(tt,ttt);
	    asf_data.export3d(fullfilename,timer_index,tt);
	  }
	else if (cmd.equals("export 2d text"))
	  {
		chooser.addChoosableFileFilter(t2dFilter);
		if(filedir == null)filedir = new File(".");
		chooser.setCurrentDirectory(filedir);
		chooser.showSaveDialog(null);
		filedir = chooser.getSelectedFile();
		
		if(filedir == null)return;
		
		String fullfilename = filedir.getAbsolutePath();
		chooser.removeChoosableFileFilter(t2dFilter);
		System.out.println(fullfilename);

		String filename = filedir.getName();
		if (filename == null) 
		  return;
		int dotindex = filename.lastIndexOf('.');
		if(dotindex == -1)
			fullfilename = fullfilename+".t2d";
		else
		{
			String fn;
		  	fn = filename.substring(dotindex+1);
			fn.toLowerCase();
			System.out.println(fn);
			if(!fn.equals("t2d"))fullfilename=fullfilename+".t2d";
		}

		disableAnim();
	    int size[] = new int[2];
	    Transform3D tr = new Transform3D();
		c.getVworldToImagePlate(tr);
		Transform3D tt = new Transform3D();
		amcTG.getTransform(tt);
		Transform3D ttt = new Transform3D();
		objTrans.getTransform(ttt);
        tr.mul(ttt);
		tr.mul(tt);
		asf_data.export2d(fullfilename,c,tr,timer_index);
		try{
				filename = fullfilename;
				int index = filename.lastIndexOf(".");
				filename = filename.substring(0,index);
				filename = filename + ".hrc";
				File outputFile = new File(filename);
				FileWriter out1 = new FileWriter(outputFile);
				String temp1="% Field Time ";
				for(int i=0;i<asf_data.number_joints;i++)
				{
					temp1 = asf_data.joints[i].name;
					for( int j=0;j<asf_data.joints[i].number_son;j++)
							temp1 = temp1 + " " + asf_data.joints[(asf_data.joints[i]).sons[j]].name;	
					temp1 = temp1+"\r";
					if(asf_data.joints[i].number_son>0)out1.write(temp1);
							temp1 = null;
				}
				out1.close();
			   }
			catch (Exception e_hrc) {
			  System.out.println("Some problem in exporting 3d file");
			  System.out.println(e_hrc.toString());
			}

	  }
    else if (cmd.equals("export image"))
	  {

	    disableAnim();
	    JpegDialog ff = new JpegDialog(this,"Jpeg");
		ff.setModal(true);
		ff.show();
	    System.out.println(ff.qua.val);

		chooser.addChoosableFileFilter(jpgFilter);
		if(filedir == null)filedir = new File(".");
		chooser.setCurrentDirectory(filedir);
		chooser.showSaveDialog(null);
		filedir = chooser.getSelectedFile();
		
		if(filedir == null)return;
		
		String fullfilename = filedir.getAbsolutePath();
		chooser.removeChoosableFileFilter(jpgFilter);
		//chooser.hide();
		System.out.println("fullfilename from ab");
		System.out.println(fullfilename);

		String filename = filedir.getName();
		if (filename == null) 
		  return;
		int dotindex = filename.lastIndexOf('.');
		if(dotindex == -1)
			fullfilename = fullfilename+".jpg";
		else
		{
			String fn;
		  	fn = filename.substring(dotindex+1);
			fn.toLowerCase();
			System.out.println(fn);
			if(!fn.equals("jpg"))fullfilename=fullfilename+".jpg";
		}
		

		full_filename = fullfilename;
		System.out.println(full_filename);
		//c.repaint();
		if(ff.current_frame)
		{
			try{
			//Thread.currentThread().sleep(2000);
			c.saveFrame(full_filename,ff.qua.val);
			}catch (Exception event_time){};
		}
		else
		{
		    if(ff.fe.value<ff.fb.value)return;
			if(asf_data.amcfilled)
			{
				int i;
				end_frame=ff.fe.value;
				frame_cur = ff.fb.value;
				if(frame_cur>=asf_data.joints[0].frame_num)return;
				if(frame_cur<0)
				{
					c.flag = false;
				    return;
				}
				images_flag=true;
				quality = ff.qua.val;
				//hideamcskeleton();
				showamc(marker_flag,ske_sel,amc_sel);
				timer.setDelay(1000);
				enableAnim();
		    }
	    }
	  }
    else if (cmd.equals("resetv")){
      objTrans.setTransform(startTrans);
	  System.out.println(startTrans);
	  rotx.val=0;
	  rotx.setText("0.0");
	  roty.val=0;
	  roty.setText("0.0");
      rotz.val=0;
	  rotz.setText("0.0");
	  tranx.val=0;
	  tranx.setText("0.0");
	  trany.val = -0.7/factor;
	  trany.setText(Double.toString(trany.val));
	  tranz.val = -0.7/factor;
	  tranz.setText(Double.toString(tranz.val));
	}
    else if (cmd.equals("resetf")) {
      resetFigure();
    }
    else if (cmd.equals("about")) {
      InfoDialog d = 
	new InfoDialog(this, "About",
		       " Mocap Viewer for viewing motion capture marker data\n"+
		       "\n Created by Wenxiang Huang, Suba Dec. 2001 \n"+
			   " Based on the java viewer program \n"+"by M. Geroch Nov. 2001");
      d.setSize(300,200);
      d.setLocation(350,300);
      d.show();
    }
	else if (cmd.equals("both"))
	{
		double angle;
		String toke;
		StringTokenizer st;
		String data[];
	    disableAnim();
		chooser.addChoosableFileFilter(bacFilter);
		if(filedir == null)filedir = new File(".");
		chooser.setCurrentDirectory(filedir);
		chooser.showOpenDialog(null);
		filedir = chooser.getSelectedFile();
		if(filedir == null)return;
		String fullfilename = filedir.getAbsolutePath();
		chooser.removeChoosableFileFilter(bacFilter);
		System.out.println(fullfilename);

		int ii=0;

		

		data = new String[100];

		String data_;

        try {

		/*File input = new File(filename);*/
		  

	      FileReader in = new FileReader(filedir);

	      BufferedReader dis = new BufferedReader(in);

		

		data_ = dis.readLine();

		if(data_ == null)return;

		int i=0;

		while(data_!=null)

		{

			data[ii] = data_;

			ii++;

			data_ = dis.readLine();

		}

		}

		catch (Exception event_obatch) {

		System.out.println("Some problem in data file -- aborting");

		System.out.println(event_obatch.toString());

		}

		

		

		

	    //pboth ff = new pboth(this,"batch processing");
		npboth ff = new npboth(this,"batch processing",chooser,asfFilter,bacFilter,amcFilter);

		ff.setModal(true);

		ff.show();

		if(!ff.do_flag)return;
	    //System.out.println(ff.qua.val);



	    String filename = ff.filename.string;

		if (filename == null) 

		  return;

		fullfilename = filename;

		

		

		String setname = fullfilename;



		int index = 0;
            int sizeofindex = 0;
		int ii_temp = ii;
		while(ii_temp>0)
		{
		   sizeofindex++;
		   ii_temp/=10;
		}

		try{

		while(index<ii)

		{

			data_ = data[index];

		      index++;

			setname = fullfilename;
			if(sizeofindex==4)
			{
			  if(index>999)
				setname = setname +"."+Integer.toString(index);
  			  else if(index>99)
				setname = setname +".0"+Integer.toString(index);
			  else if(index>9)
				setname = setname +".00"+Integer.toString(index);
			  else 
				setname = setname +".000"+Integer.toString(index);
			}
 			else if(sizeofindex == 3)
			{
			  if(index>99)
				setname = setname +Integer.toString(index);
			  else if(index>9)
				setname = setname +".0"+Integer.toString(index);
			  else 
				setname = setname +".00"+Integer.toString(index);
			}
			else if(sizeofindex == 2)
			{
			  if(index>9)
				setname = setname +Integer.toString(index);
			  else 
				setname = setname +".0"+Integer.toString(index);
			}
			else
				setname = setname +Integer.toString(index);	

			objTrans.setTransform(startTrans);

			rotx.val=0;

			rotx.setText("0.0");

			roty.val=0;

			roty.setText("0.0");

			rotz.val=0;

			rotz.setText("0.0");

			tranx.val=0;

			tranx.setText("0.0");

			trany.val = -0.7/factor;

			trany.setText(Double.toString(trany.val));

			tranz.val = -0.7/factor;

			tranz.setText(Double.toString(tranz.val));    

			

			//Tx

			st = new StringTokenizer(data_);

			toke = st.nextToken();

			angle = Double.parseDouble(toke);

			angle = angle - tranx.val;

			tranx.val += angle;

		      tranx.setText(Double.toString(tranx.val));
			Transform3D tran = new Transform3D();

			tran.set(new Vector3f((float)angle*(float)factor,0.0f, 0.0f));

			Transform3D tt = new Transform3D();

			objTrans.getTransform(tt);

			tt.mul(tran,tt);

			objTrans.setTransform(tt);

			

			//Ty

			toke = st.nextToken();

			angle = Double.parseDouble(toke);

			angle = angle - trany.val;

			trany.val += angle;

			trany.setText(Double.toString(trany.val));
			tran = new Transform3D();

			tran.set(new Vector3f(0.0f, (float)angle*(float)factor,0.0f));

			tt = new Transform3D();

			objTrans.getTransform(tt);

			tt.mul(tran,tt);

			objTrans.setTransform(tt);



			//Tz

			toke = st.nextToken();

			angle = Double.parseDouble(toke);

			angle = angle - tranz.val;

			tranz.val += angle;

			tranz.setText(Double.toString(tranz.val));
			tran = new Transform3D();

			tran.set(new Vector3f(0.0f, 0.0f,(float)angle*(float)factor));

			tt = new Transform3D();

			objTrans.getTransform(tt);

			tt.mul(tran,tt);

			objTrans.setTransform(tt);



			//x angle

			//st = new StringTokenizer(data_);

			toke = st.nextToken();

			angle = Double.parseDouble(toke);

			angle = angle - rotx.val;

			rotx.val += angle;

			rotx.setText(Double.toString(rotx.val));
		    Transform3D rot = new Transform3D();

			rot.rotX(angle*degtorad);

			tran = new Transform3D();

			objTrans.getTransform(tran);

			tran.mul(rot);

			objTrans.setTransform(tran);

			

			//y angle

			toke = st.nextToken();

			angle = Double.parseDouble(toke);

			angle = angle - roty.val;

			roty.val += angle;

			roty.setText(Double.toString(roty.val));
		    	rot = new Transform3D();

			rot.rotY(angle*degtorad);

			tran = new Transform3D();

			objTrans.getTransform(tran);

			tran.mul(rot);

			objTrans.setTransform(tran);



			//z angle

			toke = st.nextToken();

			angle = Double.parseDouble(toke);

			angle = angle - rotz.val;

			rotz.val += angle;

			rotz.setText(Double.toString(rotz.val));
		    	rot = new Transform3D();

			rot.rotZ(angle*degtorad);

			tran = new Transform3D();

			objTrans.getTransform(tran);

			tran.mul(rot);

			objTrans.setTransform(tran);



			//if(!ff.t2d_flag&&ff.image_flag)System.out.println("image only");
			String setsetname;
			if(ff.t2d_flag)

			{

				Transform3D tr = new Transform3D();

				c.getVworldToImagePlate(tr);

				tt = new Transform3D();

				amcTG.getTransform(tt);

				Transform3D ttt = new Transform3D();

				objTrans.getTransform(ttt);

				tr.mul(ttt);

				tr.mul(tt);

				setsetname = setname + ".t2d";

				//System.out.println(setname);

				asf_data.export2d(setsetname,c,tr,timer_index);	

			}
			if(ff.current_frame && ff.image_flag)

			{

				setsetname = setname + ".jpg";
				Thread.currentThread().sleep(1000);

				c.saveFrame(setsetname,ff.qua.val);

//				System.out.println("done1");

			}

			else if(ff.image_flag)

			{

				if(ff.fe.value<ff.fb.value)return;

				if(asf_data.amcfilled)

				{
					int i;
					end_frame=ff.fe.value;
					frame_cur = ff.fb.value;
					while(frame_cur<=end_frame)
					{
						if(frame_cur>=asf_data.joints[0].frame_num)continue;
						if(frame_cur<0)
						{
							continue;
						}
						if(asf_data.joints[0].frame_num>=100)
						{
							if(frame_cur<10)
								setsetname = setname +"."+ "00"+Integer.toString(frame_cur) + ".jpg";
							else if(frame_cur<100)
								setsetname = setname +"."+"0"+Integer.toString(frame_cur) + ".jpg";
							else
								setsetname = setname +"."+Integer.toString(frame_cur) + ".jpg";	
						}
						else if(asf_data.joints[0].frame_num>=10 && asf_data.joints[0].frame_num<100)
						{
							if(frame_cur<10)
								setsetname = setname +"."+"0"+Integer.toString(frame_cur) + ".jpg";
							else
								setsetname = setname +"."+Integer.toString(frame_cur) + ".jpg";	
						}
						else
							setsetname = setname +"."+Integer.toString(frame_cur) + ".jpg";
						//hideamcskeleton();
						showamc(marker_flag,ske_sel,amc_sel);
						scrollbar.setValue(frame_cur);
						field.setText(Integer.toString(frame_cur));
						frame_cur++;
						Thread.currentThread().sleep(1000);
						c.saveFrame(setsetname,ff.qua.val);
					}
				    frame_cur = end_frame;
				}
			}
		}
        }
	 	catch (Exception event_time){}

		
		objTrans.setTransform(startTrans);
		rotx.val=0;
		rotx.setText("0.0");
		roty.val=0;
		roty.setText("0.0");
		rotz.val=0;
		rotz.setText("0.0");
		tranx.val=0;
		tranx.setText("0.0");
		trany.val = -0.7/factor;
		trany.setText(Double.toString(trany.val));
		tranz.val = -0.7/factor;
		tranz.setText(Double.toString(tranz.val));    
	}
	else if(cmd.equals("m_both"))
	{
		String amcname;
		npboth ff = new npboth(this,"batch processing",chooser,asfFilter,bacFilter,amcFilter);
		ff.setModal(true);
		ff.show();
		if(!ff.do_flag)return;
		System.out.println("do_flag");
		if(!asf_data.filled)
		{
			if(ff.asffile == null)return;
		}
		if(ff.asffile != null)
		{
			 if(asf_data.filled)
					list.removeAll();
			 if(asf_data.amcfilled)
			{
					hideamcskeleton();
					int nc = amcgroup.numChildren();
					for (int j=nc-1; j >= 0; j--)  
					amcgroup.removeChild(j);
					asf_data.clearamcData();
			}
			asf_data.filled = false;
			String fullfilename = ff.asffile.getAbsolutePath();
			System.out.println(fullfilename);
			String filename = ff.asffile.getName();

			disableAnim();
			asf_data.clearData();

			 hideBbox();
		    try {
				    asf_data.setData(fullfilename,filename);
				}
			catch (Exception asfe) {
					System.out.println("loadFile: bad file reference .. aborting");
					System.out.println(asfe);
			}
		    lL1.setText(asf_data.setname);
		   if(asf_data.filled)
			{
					app = new boolean[asf_data.number_joints];
					linknet = new int[asf_data.number_joints][asf_data.number_joints-1];
					for(int i=0;i<asf_data.number_joints;i++)
					{
						for(int j=0;j<asf_data.number_joints-1;j++)
							linknet[i][j]=-1;
						app[i] = true;
					}

					for(int i=0;i<asf_data.number_joints;i++)
					{
						for(int j=0;j<asf_data.joints[i].number_son;j++)
						{
							linknet[i][j]=asf_data.joints[i].sons[j];
						}
					}
					jname = new String[asf_data.number_joints];
					for(int i=0;i<asf_data.number_joints;i++)
					{
						  jname[i] = asf_data.joints[i].name;
						  list.add(jname[i]);
					}
			}
		}
		File [] files = ff.files;
		System.out.println("files.length:"+files.length);
		String fullfilename = ff.bacfile.getAbsolutePath();
		System.out.println(fullfilename);
		int ii=0;
		String data_;
		try {
	      FileReader in = new FileReader(ff.bacfile);
	      BufferedReader dis = new BufferedReader(in);
		  data_ = dis.readLine();
 		  if(data_ == null)return;
		  while(data_!=null)
		  {
				ii++;
				data_ = dis.readLine();
		  }
		}
		catch (Exception event_obatch) {
			System.out.println("Some problem in data file -- aborting");
			System.out.println(event_obatch.toString());
		}
		String [] data = new String[ii];
		System.out.println(ii);
		ii=0;
        try {
	      FileReader in = new FileReader(ff.bacfile);
	      BufferedReader dis = new BufferedReader(in);
		  data_ = dis.readLine();
 		  if(data_ == null)return;
		  while(data_!=null)
		  {
				data[ii] = data_;
				ii++;
				data_ = dis.readLine();
		  }
		}
		catch (Exception event_obatch) {
			System.out.println("Some problem in data file -- aborting");
			System.out.println(event_obatch.toString());
		}
		String filename;
		System.out.println("files.length:"+files.length);
		System.out.println(ff.t3d_flag);
		if(!ff.t3d_flag)
		{		
			System.out.println("in procd");	
			try{
				filename = ff.savename.string+"\\"+ff.asffile.getName();
				int index = filename.lastIndexOf(".");
				filename = filename.substring(0,index);
				filename = filename + ".hrc";
				File outputFile = new File(filename);
				System.out.println("filename:"+filename);
				FileWriter out1 = new FileWriter(outputFile);
				String temp1="% Field Time ";
				System.out.println(out1);
				for(int i=0;i<asf_data.number_joints;i++)
				{
					temp1 = asf_data.joints[i].name;
					for( int j=0;j<asf_data.joints[i].number_son;j++)
							temp1 = temp1 + " " + asf_data.joints[(asf_data.joints[i]).sons[j]].name;	
					temp1 = temp1+"\r";
					if(asf_data.joints[i].number_son>0)out1.write(temp1);
							temp1 = null;
				}
				out1.close();
			   }
			catch (Exception e_hrc) {
			  System.out.println("Some problem in exporting hrc file");
			  System.out.println(e_hrc.toString());
			}
		}
		for(int i=0;i<files.length;i++)
		{
			fullfilename = files[i].getAbsolutePath();
			if(asf_data.amcfilled)
			{
				hideamcskeleton();
				int nc = amcgroup.numChildren();
				for (int j=nc-1; j >= 0; j--)  
				amcgroup.removeChild(j);
			}
			System.out.println(fullfilename);
			filename = files[i].getName();
			amcname = filename;
			int dot_index = amcname.lastIndexOf(".");
			amcname = amcname.substring(0,dot_index+1);
			System.out.println(amcname);
			disableAnim();
			asf_data.clearamcData();
			hideBbox();
			try {
			  asf_data.setamcData(fullfilename,filename);
			 }
			catch (Exception event_loadamc) {
			  System.out.println("loadFile: bad file reference .. aborting");
			  System.out.println(event_loadamc);
			}
			lL1.setText(asf_data.amcname);
			if(asf_data.amcfilled)
			{
				if(!use_currentview)
						objTrans.setTransform(startTrans);
				scrollbar.setMaximum(asf_data.frame_number);
				frame_cur=0;
				scrollbar.setValue(frame_cur);
				field.setText(Integer.toString(frame_cur));
				amcskeletonToggle.setEnabled(true);
				if(!use_currentview)
				{
					Transform3D tr = new Transform3D();
					c.getVworldToImagePlate(tr);
					Transform3D tri = new Transform3D(tr);
					tri.invert();
					int width = c.getSize().width;
					int height = c.getSize().height;
					width /= 2;
					height /= 2;
					Point3d imagePlatePoint = new Point3d();
					c.getPixelLocationInImagePlate(width,
													 height,
													 imagePlatePoint);
					tri.transform(imagePlatePoint);
					Transform3D tt = new Transform3D();
					amcTG.getTransform(tt);
					Transform3D ttt = new Transform3D();
					objTrans.getTransform(ttt);
					ttt.mul(tt);
					Point3d point = new Point3d(asf_data.joints[0].amc_position[frame_cur][0],asf_data.joints[0].amc_position[frame_cur][1],asf_data.joints[0].amc_position[frame_cur][2]);
					ttt.transform(point);
					objTrans.getTransform(ttt);
					Transform3D tran = new Transform3D();
					double x,y,z;
					x = imagePlatePoint.x-point.x;
					y = imagePlatePoint.y-point.y;
					z = imagePlatePoint.z-point.z;
					tran.set(new Vector3d(x,y,z));
					tran.mul(ttt);
					objTrans.setTransform(tran);
					Point3d point1 = new Point3d(0.0,0.0,0.0);
					tran.transform(point1);
					tranx.val=point1.x/(0.7*0.025);
					tranx.setText(Double.toString(tranx.val));
					trany.val=point1.y/(0.7*0.025);
					trany.setText(Double.toString(trany.val));
					tranz.val=point1.z/(0.7*0.025);
					tranz.setText(Double.toString(tranz.val));
				}
				if(marker_flag)
				{
					jointslon ske1 = new jointslon(asf_data.joints,asf_data.number_joints,frame_cur,app);
					amcgroup.addChild(ske1);
				}
				if(ske_sel)
				{
					skeleton_s ske2 = new skeleton_s(asf_data.joints,asf_data.number_joints,frame_cur,app,linknet);
					amcgroup.addChild(ske2);
				}
				if(amc_sel)
				{
					skeleton ske2 = new skeleton(asf_data.joints,asf_data.number_joints,frame_cur,app,linknet);
					amcgroup.addChild(ske2);
				}
				amcTG.addChild(amcgroup);
			}

			temp = file.getItem(3);
			temp.setEnabled(true);
			temp = file.getItem(4);
			temp.setEnabled(true);
			temp = file.getItem(5);
			temp.setEnabled(true);

		
		  //do processing
			{
			double angle;
			String toke;
			StringTokenizer st;
			disableAnim();

			filename = ff.savename.string;
			System.out.println("ff.savename.string: "+filename);
			System.out.println("amcname: "+amcname);
			if (filename == null) 
			{
				filename = ff.filename.string;
				if(filename == null)
					filename = amcname;
				else
					filename = filename + "." + amcname;
			}
			else
			{
				System.out.println(ff.filename.string);
				if(ff.filename.string.equals(""))
					filename = filename+"\\"+amcname;
				else
					filename = filename+"\\"+ff.filename.string+"."+amcname;
			}
			
			fullfilename = filename;
			System.out.println(fullfilename);
			String setname = fullfilename;
			int index = 0;
			int sizeofindex = 0;
			int ii_temp = ii;
			while(ii_temp>0)
			{
			   sizeofindex++;
			   ii_temp/=10;
			}

			String setsetname;
			if(ff.t3d_flag)
			{
				disableAnim();
				Transform3D tt = new Transform3D();
				amcTG.getTransform(tt);
				Transform3D ttt = new Transform3D();
				objTrans.getTransform(ttt);
				tt.mul(tt,ttt);
				setsetname = setname + "t3d";
				asf_data.export3d(setsetname,timer_index,tt);
			}


		try{

		while(index<ii)

		{

			data_ = data[index];

		      index++;

			setname = fullfilename;
			if(sizeofindex==4)
			{
			  if(index>999)
				setname = setname +Integer.toString(index);
  			  else if(index>99)
				setname = setname +"0"+Integer.toString(index);
			  else if(index>9)
				setname = setname +"00"+Integer.toString(index);
			  else 
				setname = setname +"000"+Integer.toString(index);
			}
 			else if(sizeofindex == 3)
			{
			  if(index>99)
				setname = setname +Integer.toString(index);
			  else if(index>9)
				setname = setname +"0"+Integer.toString(index);
			  else 
				setname = setname +"00"+Integer.toString(index);
			}
			else if(sizeofindex == 2)
			{
			  if(index>9)
				setname = setname +Integer.toString(index);
			  else 
				setname = setname +"0"+Integer.toString(index);
			}
			else
				setname = setname +Integer.toString(index);	
			objTrans.setTransform(startTrans);
			rotx.val=0;
			rotx.setText("0.0");
			roty.val=0;
			roty.setText("0.0");
			rotz.val=0;
			rotz.setText("0.0");
			tranx.val=0;
			tranx.setText("0.0");
			trany.val = -0.7/factor;
			trany.setText(Double.toString(trany.val));
			tranz.val = -0.7/factor;
			tranz.setText(Double.toString(tranz.val));    
	
			//Tx
			st = new StringTokenizer(data_);
			toke = st.nextToken();
			angle = Double.parseDouble(toke);
			angle = angle - tranx.val;
			tranx.val += angle;
		    tranx.setText(Double.toString(tranx.val));
			Transform3D tran = new Transform3D();
			tran.set(new Vector3f((float)angle*(float)factor,0.0f, 0.0f));
			Transform3D tt = new Transform3D();
			objTrans.getTransform(tt);
			tt.mul(tran,tt);
			objTrans.setTransform(tt);

			

			//Ty
			toke = st.nextToken();
			angle = Double.parseDouble(toke);
			angle = angle - trany.val;
			trany.val += angle;
			trany.setText(Double.toString(trany.val));
			tran = new Transform3D();
			tran.set(new Vector3f(0.0f, (float)angle*(float)factor,0.0f));
			tt = new Transform3D();
			objTrans.getTransform(tt);
			tt.mul(tran,tt);
			objTrans.setTransform(tt);


			//Tz
			toke = st.nextToken();
			angle = Double.parseDouble(toke);
			angle = angle - tranz.val;
			tranz.val += angle;
			tranz.setText(Double.toString(tranz.val));
			tran = new Transform3D();
			tran.set(new Vector3f(0.0f, 0.0f,(float)angle*(float)factor));
			tt = new Transform3D();
			objTrans.getTransform(tt);
			tt.mul(tran,tt);
			objTrans.setTransform(tt);



			//x angle

			//st = new StringTokenizer(data_);

			toke = st.nextToken();

			angle = Double.parseDouble(toke);

			angle = angle - rotx.val;

			rotx.val += angle;

			rotx.setText(Double.toString(rotx.val));
		    Transform3D rot = new Transform3D();

			rot.rotX(angle*degtorad);

			tran = new Transform3D();

			objTrans.getTransform(tran);

			tran.mul(rot);

			objTrans.setTransform(tran);
			//y angle
			toke = st.nextToken();
			angle = Double.parseDouble(toke);
			angle = angle - roty.val;
			roty.val += angle;
			roty.setText(Double.toString(roty.val));
		    	rot = new Transform3D();
			rot.rotY(angle*degtorad);
			tran = new Transform3D();
			objTrans.getTransform(tran);
			tran.mul(rot);
			objTrans.setTransform(tran);
			//z angle
			toke = st.nextToken();
			angle = Double.parseDouble(toke);
			angle = angle - rotz.val;
			rotz.val += angle;
			rotz.setText(Double.toString(rotz.val));
		    	rot = new Transform3D();
			rot.rotZ(angle*degtorad);
			tran = new Transform3D();
			objTrans.getTransform(tran);
			tran.mul(rot);
			objTrans.setTransform(tran);
			if(ff.t2d_flag)
			{				
				Transform3D tr = new Transform3D();
				c.getVworldToImagePlate(tr);
				tt = new Transform3D();
				amcTG.getTransform(tt);
				Transform3D ttt = new Transform3D();
				objTrans.getTransform(ttt);
				tr.mul(ttt);
				tr.mul(tt);
				setsetname = setname + ".t2d";
				//System.out.println(setname);
				asf_data.export2d(setsetname,c,tr,timer_index);	
			}
			if(ff.current_frame && ff.image_flag)

			{

				setsetname = setname + ".jpg";
				Thread.currentThread().sleep(1000);

				c.saveFrame(setsetname,ff.qua.val);

//				System.out.println("done1");

			}

			else if(ff.image_flag)

			{

				if(ff.fe.value<ff.fb.value)return;

				if(asf_data.amcfilled)

				{
					end_frame=ff.fe.value;

					frame_cur = ff.fb.value;

					while(frame_cur<=end_frame)

					{

						if(frame_cur>=asf_data.joints[0].frame_num)continue;

						if(frame_cur<0)

						{
							continue;
						}
						if(asf_data.joints[0].frame_num>=100)
						{
							if(frame_cur<10)
								setsetname = setname +"."+ "00"+Integer.toString(frame_cur) + ".jpg";
							else if(frame_cur<100)
								setsetname = setname +"."+"0"+Integer.toString(frame_cur) + ".jpg";
							else
								setsetname = setname +"."+Integer.toString(frame_cur) + ".jpg";	
						}
						else if(asf_data.joints[0].frame_num>=10 && asf_data.joints[0].frame_num<100)
						{
							if(frame_cur<10)
								setsetname = setname +"."+"0"+Integer.toString(frame_cur) + ".jpg";
							else
								setsetname = setname +"."+Integer.toString(frame_cur) + ".jpg";	
						}
						else
							setsetname = setname +"."+Integer.toString(frame_cur) + ".jpg";
						//hideamcskeleton();
						showamc(marker_flag,ske_sel,amc_sel);
						scrollbar.setValue(frame_cur);
						field.setText(Integer.toString(frame_cur));
						frame_cur++;
						Thread.currentThread().sleep(1000);
						c.saveFrame(setsetname,ff.qua.val);
					}
				    frame_cur = end_frame;
				}
			}
		}
        }
	 	catch (Exception event_time){}
		objTrans.setTransform(startTrans);
		rotx.val=0;
		rotx.setText("0.0");
		roty.val=0;
		roty.setText("0.0");
		rotz.val=0;
		rotz.setText("0.0");
		tranx.val=0;
		tranx.setText("0.0");
		trany.val = -0.7/factor;
		trany.setText(Double.toString(trany.val));
		tranz.val = -0.7/factor;
		tranz.setText(Double.toString(tranz.val));    
		}
		}

	}
	else
	{
		anit();
	}

    
  } // end of actionPerformed()
  
  
  // -------------- response to checking an item
  public void itemStateChanged(ItemEvent e) {
    if (e.getSource() == axisToggle) {
      if (e.getStateChange() == ItemEvent.SELECTED) 
	//objTrans.addChild(axisTG);

	  axisTG.addChild(axisRoot);
      else 
	  hideAxis();
    }

	else if (e.getSource() == setdefaultview){

     if (e.getStateChange() == ItemEvent.SELECTED) 

	  use_currentview = true;  

      else 

	  use_currentview = false;

    }
    else if (e.getSource() == floorToggle) {
      if (e.getStateChange() == ItemEvent.SELECTED) 
	objTrans.addChild(floorRoot);
      else 
	hideFloor();
    }
	else if (e.getSource() == linktoggle)
	{
	   if (e.getStateChange() == ItemEvent.SELECTED)
	  {
	    ((CheckboxMenuItem)func.getItem(1)).setState(false);
		System.out.println("hit link toggle");
		//hide_showtoggle.setState(false);
		link=true;
		hide_show=false;



  	      linkselected = list.getSelectedIndexes();

		// deselect everything

		for (int i=0; i<linkselected.length; i++)

			list.deselect(linkselected[i]);

		// select everything in hideshowselected

	      for (int i=0; i<hideshowselected.length; i++)

		      list.select(hideshowselected[i]);

	      




	  }
	  else
	  {
		link = false;
	  }
	}
	else if (e.getSource() == hide_showtoggle)
	{
	   if (e.getStateChange() == ItemEvent.SELECTED)
	  {
		linktoggle.setState(false);
		((CheckboxMenuItem)func.getItem(0)).setState(false);
		link=false;
		hide_show=true;



	      hideshowselected = list.getSelectedIndexes();

		// deselect everything

		for (int i=0; i<hideshowselected.length; i++)

			list.deselect(hideshowselected[i]);

		// select everything in linkselected

	      for (int i=0; i<linkselected.length; i++)

		      list.select(linkselected[i]);

	      




	  }
	  else
		hide_show=false;
	}
	else if (e.getSource() == markeronlyToggle) {
	  if (e.getStateChange() == ItemEvent.SELECTED)
	  {
	    marker_flag = true;
	  }
	  else
	  {
	    marker_flag = false;
	  }
	  //hideamcskeleton();
	  showamc(marker_flag,ske_sel,amc_sel);
	}
	else if (e.getSource() == skeletonToggle) {
	  System.out.println("hit");
      if (e.getStateChange() == ItemEvent.SELECTED)
	  {
		ske_sel = true;
		amcskeletonToggle.setState(false);
		amc_sel = false;
		//hideamcskeleton();
		showamc(marker_flag,ske_sel,amc_sel);
	  } 
	  else 
	  {
		ske_sel = false;
		//hideamcskeleton();
		showamc(marker_flag,ske_sel,amc_sel);
	  }
    }
	else if (e.getSource() == amcskeletonToggle) {
	  if (e.getStateChange() == ItemEvent.SELECTED)
	  {
		amc_sel = true;           
		skeletonToggle.setState(false);
		ske_sel = false;
		//hideamcskeleton();
		showamc(marker_flag,ske_sel,amc_sel);
	  } 
	  else 
	  {
		//hideamcskeleton();
		amc_sel = false;
		showamc(marker_flag,ske_sel,amc_sel);
	  }
    }
    else if (e.getSource() == bboxToggle) {
      if (e.getStateChange() == ItemEvent.SELECTED) 
	objTrans.addChild(bboxRoot);
      else 
	hideBbox();
    }	
  } // end of itemStateChanged()
  
  
  
  
  // ************************************* 
  // remaining part of constructor for the Viewer class
  // ***************************************
  public void dotheRest() {
    // set up a starting transformation
	System.out.println("in dotheRest");
    Transform3D trtemp = new Transform3D();
    trtemp.set(new Vector3f(0.0f, -1.0f, -1.0f));
    Transform3D rotemp = new Transform3D();
    rotemp.rotY(Math.PI/2.0d - 0.3d);
    Transform3D sctemp = new Transform3D();
    sctemp.setScale(0.7d);
    startTrans = new Transform3D();
    startTrans.setIdentity();
    //startTrans.mul(rotemp);
    startTrans.mul(sctemp);
    startTrans.mul(trtemp);
    
    thisData = new Data();                     
	asf_data = new Data_asf();
    //thisData.setData("Data/megmoves02.txt");
	
    createContents();
    
    // ---- Create a virtual universe, moving view platform back
    u = new SimpleUniverse(c);
    u.getViewingPlatform().setNominalViewingTransform();

	  
    // --- create a simple scene and attach it to the universe
    BranchGroup scene = createSceneGraph();
    scene.compile();
    u.addBranchGraph(scene);
    View view = u.getViewer().getView();
    System.out.println((view.getFieldOfView()/2.0)*180.0/Math.PI);
    System.out.println(view.getFrontClipDistance());
    System.out.println(view.getBackClipDistance());
  }
  
  
  
  
  // **************************************************
  // **************************************************
  // The following allows Viewer to be run as an application
  // as well as an applet
  // ---------------------------------------------
  public static void main(String[] args) {
    try {
      //new MainFrame(new Viewer(), 800, 512);                       
	  System.out.println("in main");
      Viewer mcv = new Viewer(1000, 512);
      mcv.setTitle("Mocap Viewer");
      mcv.setVisible(true);
    }
    catch (Exception e) {
      System.out.println("something fishy in main..."+e);
    }
  }
  class FilePreviewer extends JComponent implements PropertyChangeListener {
	ImageIcon thumbnail = null;
	File f = null;

	public FilePreviewer(JFileChooser fc) {
	    setPreferredSize(new Dimension(100, 50));
	    fc.addPropertyChangeListener(this);
	}

	public void loadImage() {
	    if(f != null) {
		ImageIcon tmpIcon = new ImageIcon(f.getPath());
		if(tmpIcon.getIconWidth() > 90) {
		    thumbnail = new ImageIcon(
			tmpIcon.getImage().getScaledInstance(90, -1, Image.SCALE_DEFAULT));
		} else {
		    thumbnail = tmpIcon;
		}
	    }
	}

	public void propertyChange(PropertyChangeEvent e) {
	    String prop = e.getPropertyName();
	    if(prop == JFileChooser.SELECTED_FILE_CHANGED_PROPERTY) {
		f = (File) e.getNewValue();
		if(isShowing()) {
		    loadImage();
		    repaint();
		}
	    }
	}

	public void paint(Graphics g) {
	    if(thumbnail == null) {
		loadImage();
	    }
	    if(thumbnail != null) {
		int x = getWidth()/2 - thumbnail.getIconWidth()/2;
		int y = getHeight()/2 - thumbnail.getIconHeight()/2;
		if(y < 0) {
		    y = 0;
		}

		if(x < 5) {
		    x = 5;
		}
		thumbnail.paintIcon(this, g, x, y);
	    }
	}
    }
}


















